/*
 * goben.c
 *
 * Version 0.0.1, Wayne Myers, Thu Mar 13 16:58:19 2003
 */

#include <gtk/gtk.h>
#include <unistd.h> /* sleep */
#include <signal.h>   /* SIGCHLD catching stuff */
#include <sys/types.h>
#include <sys/wait.h>

#include "goben.h"
#include "gui_menu.h"
#include "gui_goboard.h"
#include "gui_dialogs.h"
#include "game.h"
#include "cfg.h"

struct game *g;

/* sigchld_handler
 * by Eric Moew, from GTK+ FAQ
 * Because we need to know what happens to our children.
 */
void sigchld_handler(int num)
{

	sigset_t set, oldset;
	pid_t pid;
	int status, exitstatus;
	
	/* block other incoming SIGCHLD signals */
	sigemptyset(&set);
	sigaddset(&set, SIGCHLD);
	sigprocmask(SIG_BLOCK, &set, &oldset);
	
	/* wait for child */
	while((pid = waitpid((pid_t)-1, &status, WNOHANG)) > 0)
		{
			if(WIFEXITED(status))
				{
					exitstatus = WEXITSTATUS(status);
					
					/*g_warning("goben: child exited, pid = %d, exit status = %d\n", */
					/*	(int)pid, exitstatus); */
				}
			else if(WIFSIGNALED(status))
				{
					exitstatus = WTERMSIG(status);
					
					g_warning("goben: child terminated by signal %d, pid = %d\n",
						exitstatus, (int)pid);
				}
			else if(WIFSTOPPED(status))
				{
					exitstatus = WSTOPSIG(status);
					
					g_warning("goben: child stopped by signal %d, pid = %d\n",
						exitstatus, (int)pid);
				}
			else
				{
					g_warning("goben: child exited magically, pid = %d\n",
						(int)pid);
				}
		}
	
	/* re-install the signal handler (some systems need this) */
	signal(SIGCHLD, sigchld_handler);
	
	/* and unblock it */
	sigemptyset(&set);
	sigaddset(&set, SIGCHLD);
	sigprocmask(SIG_UNBLOCK, &set, &oldset);
}

void save_settings_callback(GtkWidget *w, gpointer data)
{
	save_cfg(g);
	g->need_save = 0;
	gtk_widget_destroy(GTK_WIDGET(data));
}

void dont_save_callback(GtkWidget *w, gpointer data)
{
	g->need_save = 0;
	gtk_widget_destroy(GTK_WIDGET(data));
}

gint destroyapp(GtkWidget *widget, gpointer data)
{

        g_print("Sayonara...\n");

	/* Pop up save settings dialog, maybe*/
	if(g->need_save == 1) {
		
		yesno_dialog("Settings not saved. Save now?",
			     save_settings_callback, dont_save_callback);

		while(g->need_save == 1) {
			/* wait for answer */
			prod_gtk();
		}

	}

	/* free data */
	free_game(g);

	/* end gtk */
	gtk_main_quit();

	/* there's a reason why this returns FALSE, but I forget what it is */
	return (FALSE);

}

int main ( int argc, char *argv[]) {

	GtkWidget *window;
	GtkWidget *vbox;
	
	/* Initialise gtk */
	gtk_set_locale ();
	gtk_init (&argc, &argv);
	gtk_rc_parse (GOBEN_RC);
	
	/* Initialise game */
	g = new_game();

	/* Main Window */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(destroyapp), NULL);
	gtk_window_set_title(GTK_WINDOW(window), NAME " v" VERSION);
	gtk_container_border_width(GTK_CONTAINER(window), 0);
	gtk_widget_set_usize(GTK_WIDGET(window), APP_HSIZE, APP_VSIZE);
	
	/* Vbox for widgets */
	vbox = gtk_vbox_new(FALSE, 1);
        
	/* Border width is 1 */
	gtk_container_border_width(GTK_CONTAINER(vbox), 1);
        
	/* Add vbox to mainwindow */
	gtk_container_add(GTK_CONTAINER(window), vbox);
        
	/* Make menu */
	create_main_menu(window, vbox, g);
	
	/* Add go board */
	create_goban(vbox, g);
	
	/* Make stuff visible */
	gtk_widget_show(vbox);
	gtk_widget_show(window);


	/* install a signal handler for SIGCHLD signals */
        signal(SIGCHLD, sigchld_handler);

	/* Go */
	gtk_main();
	
	return(0);
	
}
