/*
 * gui_actions.c
 */

#include "goben.h"
#include "gui_dialogs.h"
#include "gui_goboard.h"
#include "sgf.h"
#include "gtkgoboard.h"
#include "aihandle.h"

/* action_new
 * resets everything for new game
 *
 * FIXME restores default game settings for now,
 * but shouldn't muck with settings here
 * later new_game() will look up settings from file
 * and we will be able to save settings
 * HMM - not sure. expected behaviour should be 'a new game,
 * with exactly whatever settings I just set for the last one,
 * and never mind what my saved defaults may be...'
 */
void action_new(GtkWidget *widget, gpointer data)
{
	struct game *g;

	g = (struct game *)data;

	/* Not in the middle of a game, please */
	if((g->mode == GOBAN_PLAYH) ||
	   (g->mode == GOBAN_PLAYC) ||
	   (g->mode == GOBAN_PLAYW)) {
		message_dialog("Sorry", "Please stop the current game before starting a new one\n");
		return;
	}

	/* initialise data with prejudice */
	g->mode = GOBAN_EDIT;
	cleanup_ai(g);
	init_game(&g, 0);
	set_current(g->col->tree->nodes, 1);
	set_edit_buttons();
	init_goban(g);
	update_soft_settings();
	set_user_strings(g);
	sync_gui_board(g);
	update_state(g);

}

/* action_open */
void action_open(GtkWidget *widget, gpointer data)
{
	file_dialog("Load", FILE_LOAD, data);
}

/* action_save */
void action_save(GtkWidget *widget, gpointer data)
{
	file_dialog("Save", FILE_SAVE, data);
//	save_sgf(((struct game *)data)->col);
}

/* action_save_as */
void action_save_as(GtkWidget *widget, gpointer data)
{
	file_dialog("Save As", FILE_SAVEAS, data);
}



/* action_quit */
void action_quit(GtkWidget *widget, gpointer data)
{
        destroyapp(widget, data);
}



/* action_about */
void action_about(GtkWidget *widget, gpointer data)
{
	message_dialog("About...",
		       NAME " version " VERSION
		       "\n by " AUTHOR "\n" EMAIL
		);
}
