/*
 * gui_dialogs.c
 */

#include <gtk/gtk.h>
#include "game.h"
#include "gui_dialogs.h"
#include "sgf.h"
#include "gui_goboard.h"
#include "gui_actions.h"
#include "cfg.h"

/*
 * close_message_dialog
 *
 * Handles normal close of dialog box
 *
 */ 
void close_dialog(GtkWidget *widget, gpointer data)
{

        GtkWidget *dialog_widget  = (GtkWidget *) data;
        
        gtk_widget_destroy(dialog_widget);
        
}

/*
 * clear_message_dialog
 *
 * Handles abnormal close of dialogbox
 */
void clear_dialog(GtkWidget *widget, gpointer data)
{
        gtk_grab_remove(widget);
}


/* info_dialog
 * give it a game tree
 * and a game
 * fills out info in dialog
 * displays it
 */
void info_dialog(struct game *g, struct sgftree *tree)
{

	struct sgfxtra *xtra;

        GtkWidget *label;
        GtkWidget *button;
        GtkWidget *info_window;
        
	GtkWidget *frame;
	GtkWidget *box, *lbox, *mbox, *rbox, *hbox, *vvhbox;
	GtkWidget *entry, *text, *scroll;

	GString *handicap;
	GString *black;
	GString *white;

	/* fill out xtra struct */
	xtra = get_xtra(tree);

        /* create dialog window */
        info_window = gtk_dialog_new();
        
        gtk_signal_connect(GTK_OBJECT(info_window), "destroy",
                           GTK_SIGNAL_FUNC(clear_dialog),
                           NULL);
        
        gtk_window_set_title(GTK_WINDOW(info_window), "Game Info");
        gtk_container_border_width(GTK_CONTAINER(info_window), 0);
        
        /* create ok button with focus */
        button = gtk_button_new_with_label("OK");
        gtk_signal_connect(GTK_OBJECT(button), "clicked",
                           GTK_SIGNAL_FUNC(close_dialog),
                           info_window);
        
        GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(info_window)->action_area),
                           button, TRUE, TRUE, 0);
        gtk_widget_grab_default(button);
        gtk_widget_show(button);
        
	/* Three columns (vboxes) in an hbox  */
	hbox = gtk_hbox_new(TRUE, 2);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(info_window)->vbox), hbox, FALSE, FALSE, 0);

	lbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), lbox, FALSE, FALSE, 0);
	mbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), mbox, FALSE, FALSE, 0);
	rbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), rbox, FALSE, FALSE, 0);


	/* frame for players info BT, PB, BR, WT, PW, WR*/

	frame = gtk_frame_new("Players:");
	gtk_box_pack_start(GTK_BOX(lbox), frame, FALSE, FALSE, 0);
	box = gtk_vbox_new (TRUE, 2);
	gtk_container_add (GTK_CONTAINER (frame), box);

	black = g_string_new(g->pname[BLACK]->str);
	if(xtra->rank[BLACK]) {
		g_string_sprintfa(black, " %s", xtra->rank[BLACK]->str);
	}
	if(xtra->team[BLACK]) {
		g_string_sprintfa(black, " (%s)", xtra->team[BLACK]->str);
	}

	white = g_string_new(g->pname[WHITE]->str);
	if(xtra->rank[WHITE]) {
		g_string_sprintfa(white, " %s", xtra->rank[WHITE]->str);
	}
	if(xtra->team[WHITE]) {
		g_string_sprintfa(black, " (%s)", xtra->team[WHITE]->str);
	}

	label = gtk_label_new("Black:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(black->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

	label = gtk_label_new("White:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(white->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

	g_string_free(black, TRUE);
	g_string_free(white, TRUE);

        gtk_widget_show(box);
        gtk_widget_show(frame);

	/* frame for game info - GN, ON, GC */
	frame = gtk_frame_new("Game:");
	gtk_box_pack_start(GTK_BOX(lbox), frame, FALSE, FALSE, 0);
	box = gtk_vbox_new (FALSE, 2);
	gtk_container_add (GTK_CONTAINER (frame), box);

	label = gtk_label_new("Game Name:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(g->info) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(g->info->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
	gtk_widget_show(entry);

	label = gtk_label_new("Opening:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->opening) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->opening->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
	gtk_widget_show(entry);

	label = gtk_label_new("Game info:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	/* scrolled text widget for this */
	vvhbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), vvhbox, TRUE, TRUE, 0);
	/* set an arbitrary height of 64 pixels for text box */
	gtk_widget_set_usize(vvhbox, 0, 64);
	/* ye text widget */
	text = gtk_text_new(NULL, NULL);
	/* wrap everything, but don't let us edit yet */
	gtk_text_set_editable(GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
	gtk_text_set_line_wrap(GTK_TEXT(text), TRUE);
	gtk_box_pack_start(GTK_BOX (vvhbox), text, TRUE, TRUE, 0);
	gtk_widget_show(text);
	/* add a scrollbar on the right */
	scroll = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
	gtk_box_pack_start(GTK_BOX(vvhbox), scroll, TRUE, TRUE, 0);
	gtk_widget_show(scroll);
	if(xtra->gameinfo) {
	/* add text */
		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, xtra->gameinfo->str, -1);
		/* set position to beginning of text */
		gtk_editable_set_position(GTK_EDITABLE(text), 0);
	}

	gtk_widget_show(text);
	gtk_widget_show(scroll);
	gtk_widget_show(vvhbox);

        gtk_widget_show(box);
        gtk_widget_show(frame);



	/* frame for match info - EV, DT, PC, RE, RO */

	frame = gtk_frame_new("Match:");
	gtk_box_pack_start(GTK_BOX(mbox), frame, TRUE, TRUE, 0);
	box = gtk_vbox_new (TRUE, 2);
	gtk_container_add (GTK_CONTAINER (frame), box);

	label = gtk_label_new("Result:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(g->score) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(g->score->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);


	label = gtk_label_new("Event:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->event) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->event->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

	label = gtk_label_new("Round:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->round) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->round->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

	label = gtk_label_new("Date:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->date) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->date->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

	label = gtk_label_new("Place:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->place) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->place->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

        gtk_widget_show(box);
        gtk_widget_show(frame);

	/* frame for rules info - HA, KM, OT, RU, TM */
	/* FIXME - RU, TM not handled yet */
	frame = gtk_frame_new("Rules:");
	gtk_box_pack_start(GTK_BOX(rbox), frame, FALSE, FALSE, 0);
	box = gtk_vbox_new (TRUE, 2);
	gtk_container_add (GTK_CONTAINER (frame), box);

	handicap = g_string_new(NULL);

	g_string_sprintf(handicap, "Komi: %.1f, Handicap: %d", g->komi, g->handicap);

	label = gtk_label_new(g_strdup(handicap->str));
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	
	g_string_free(handicap, TRUE);

	label = gtk_label_new("Byo-yomi:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->overtime) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->overtime->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);

        gtk_widget_show(box);
        gtk_widget_show(frame);

	/* frame for misc info - AN, SO, CP, US */
	frame = gtk_frame_new("Misc:");
	gtk_box_pack_start(GTK_BOX(rbox), frame, FALSE, FALSE, 0);
	box = gtk_vbox_new (TRUE, 2);
	gtk_container_add (GTK_CONTAINER (frame), box);


	label = gtk_label_new("Annotator:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->annotator) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->annotator->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);
	
	
	label = gtk_label_new("Source:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->source) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->source->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);
	
	
	label = gtk_label_new("Copyright:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->copyright) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->copyright->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);
	
	
	label = gtk_label_new("User:");
//        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
	gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	entry = gtk_entry_new();
	if(xtra->user) 
		gtk_entry_set_text(GTK_ENTRY(entry), g_strdup(xtra->user->str));
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, TRUE, 0);
	gtk_widget_show(entry);
	
	gtk_widget_show(box);
	gtk_widget_show(frame);

	/* dialog outro */
	gtk_widget_show(mbox);
        gtk_widget_show(lbox);
	gtk_widget_show(rbox);
	gtk_widget_show(hbox);

        /* show dialog box and grab focus */
        gtk_widget_show(info_window);
        gtk_grab_add(info_window);

	/* free xtra struct */
	free_xtra(xtra);

}


/* File selector dialog */
GtkWidget *fd;

void do_load(GtkWidget *w, gpointer data)
{
	gchar *filename;
	struct game *g;
	struct sgfnode *node;

	g = (struct game *)data;

	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fd));

	
	if(load_sgf(g, filename)) {
		/* Error loading, pop up warning message, clean up */
		message_dialog("Sorry", "Load file failed\n");

		/* Reset, as status of data now uncertain */
		action_new(NULL, (gpointer)g);

	} else {

		/* load_sgf loads sgf file only,
		 * now get the game struct in sync
		 */
		/* get number of moves in main variation */
		init_game(&g, 1);
		g->num_moves = sgf_movecount(g, g->col->tree->nodes);
		/* rewind to beginning again */
		node = g->col->tree->nodes;
		/* set point to first variation of last node found */
		set_current(node, 1);
		/* get variations counted if any (shouldn't be but...) */
		update_vars(node, 1);
		/* now sync gui with that */
		g->mode = GOBAN_EDIT;
		init_goban(g);
		update_soft_settings();
		set_user_strings(g);
		sync_gui_board(g);
		update_state(g);
		set_edit_buttons();
		set_status_bar(NULL);
		set_text(g->comment->str);
	}

}

void do_save(GtkWidget *w, gpointer data)
{

	gchar *filename;
	struct game *g;

	g = (struct game *)data;

	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fd));
	
	if(save_sgf_as(g->col, filename)) {
		/* Error loading, pop up warning message, clean up */
		message_dialog("Sorry", "Save file failed\n");
	}


}

/* file_dialog
 * handle loading, saving, saving as
 */
void file_dialog(char *title, gint type, gpointer data)
{

	struct game *g;

	g = (struct game *)data;

	if((type == FILE_LOAD) && (g->mode != GOBAN_EDIT)) {
		message_dialog("Sorry", "You can only load files in edit mode\n");
		return;
	}

	fd = gtk_file_selection_new(title);

	gtk_file_selection_set_filename(
		GTK_FILE_SELECTION(fd),
		g->col->filename);

	if(type == FILE_LOAD) {
		   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(fd)->ok_button),
   			   "clicked", GTK_SIGNAL_FUNC (do_load), data);
	} else {
		   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(fd)->ok_button),
   			   "clicked", GTK_SIGNAL_FUNC (do_save), data);
	}

	/* handle abnormal close */
        gtk_signal_connect(GTK_OBJECT(fd), "destroy",
                           GTK_SIGNAL_FUNC(clear_dialog),
                           NULL);

	/* ok, cancel buttons */
	gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(fd)->cancel_button),
		"clicked",
		(GtkSignalFunc) gtk_widget_destroy,
		GTK_OBJECT(fd));
	gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(fd)->ok_button),
		"clicked",
		(GtkSignalFunc) gtk_widget_destroy,
		GTK_OBJECT(fd));


        /* show dialog box and grab focus */
        gtk_widget_show(fd);
        gtk_grab_add(fd);

}

/* message_dialog
 *
 * A simple pop up dialog with an OK button
 */
void message_dialog(char *title, char *message)
{
        GtkWidget *label;
        GtkWidget *button;
        GtkWidget *dialog_window;
        
        /* create dialog window */
        dialog_window = gtk_dialog_new();
        
        gtk_signal_connect(GTK_OBJECT(dialog_window), "destroy",
                           GTK_SIGNAL_FUNC(clear_dialog),
                           NULL);
        
        gtk_window_set_title(GTK_WINDOW(dialog_window), title);
        gtk_container_border_width(GTK_CONTAINER(dialog_window), 0);
        
        /* create ok button with focus */
        button = gtk_button_new_with_label("OK");
        gtk_signal_connect(GTK_OBJECT(button), "clicked",
                           GTK_SIGNAL_FUNC(close_dialog),
                           dialog_window);
        
        GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->action_area),
                           button, TRUE, TRUE, 0);
        gtk_widget_grab_default(button);
        gtk_widget_show(button);
        
        /* label */
        label = gtk_label_new(message);
        
        /* pad label text */
        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
        
        /* add label */
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox),
                           label, TRUE, TRUE, 0);
        
        /* show it */
        gtk_widget_show(label);
        
        /* show dialog box and grab focus */
        gtk_widget_show(dialog_window);
        gtk_grab_add(dialog_window);
        
}


void yesno_dialog(char *message, void (*yesfunc)(GtkWidget *w, gpointer data),
		  void (*nofunc)(GtkWidget *w, gpointer data))
{

        GtkWidget *label;
        GtkWidget *yes_button;
	GtkWidget *no_button;
        GtkWidget *dialog_window;
        
        /* create dialog window */
        dialog_window = gtk_dialog_new();
        
        gtk_signal_connect(GTK_OBJECT(dialog_window), "destroy",
                           GTK_SIGNAL_FUNC(clear_dialog),
                           NULL);
        
        gtk_window_set_title(GTK_WINDOW(dialog_window), "Question:");
        gtk_container_border_width(GTK_CONTAINER(dialog_window), 0);
        
        /* create yes button with focus */
        yes_button = gtk_button_new_with_label("Yes");
        gtk_signal_connect(GTK_OBJECT(yes_button), "clicked",
			   GTK_SIGNAL_FUNC(yesfunc),
                           dialog_window);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->action_area),
                           yes_button, TRUE, TRUE, 0);
        gtk_widget_show(yes_button);
        
        /* create no button with focus */
        no_button = gtk_button_new_with_label("No");
        gtk_signal_connect(GTK_OBJECT(no_button), "clicked",
                           GTK_SIGNAL_FUNC(nofunc),
                           dialog_window);
        GTK_WIDGET_SET_FLAGS(no_button, GTK_CAN_DEFAULT);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->action_area),
                           no_button, TRUE, TRUE, 0);
        gtk_widget_grab_default(no_button);
        gtk_widget_show(no_button);

        /* label */
        label = gtk_label_new(message);
        
        /* pad label text */
        gtk_misc_set_padding(GTK_MISC(label), 10, 10);
        
        /* add label */
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox),
                           label, TRUE, TRUE, 0);
        
        /* show it */
        gtk_widget_show(label);
        
        /* show dialog box and grab focus */
        gtk_widget_show(dialog_window);
        gtk_grab_add(dialog_window);

}
