/* gui_goboard.c --

 * Derived from testgoboard.c by Changwoo Ryu

   Copyright (C) 1998 Copyright (C) 1998 Changwoo Ryu

   This program is free software; you can redistribute it and'or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
   USA */

/* Written by Changwoo Ryu <cwryu@adam.kaist.ac.kr>. */

#include <gtk/gtk.h>
#include "gtkgoboard.h"
#include "gui_goboard.h"
#include "gui_actions.h" /* not_implemented */
#include "gui_dialogs.h"  /* message_dialog */
#include "gui_settings.h" /* settings dialog */
#include "game.h"
#include "aihandle.h"
#include "sgf.h"



/* callbacks for game board */
static void pressed_callback (GtkGoBoard *board, gint x, gint y, gpointer data);
static void enter_callback (GtkGoBoard *board, gint x, gint y, gpointer data);
static void leave_callback (GtkGoBoard *board, gint x, gint y, gpointer data);

/* callbacks for buttons */
static void play_callback(GtkWidget *button, gpointer data);
static void pass_callback(GtkWidget *button, gpointer data);
static void resign_callback(GtkWidget *button, gpointer data);

/* the goban */
GtkWidget *gframe;
GtkWidget *go_board;

/* widgets with user settable strings */
GtkWidget *uframe;
GtkWidget *p1entry;
GtkWidget *p2entry;

/* control buttons */
GtkWidget *play_button;
GtkWidget *pass_button;
GtkWidget *resign_button;

/* sgfnav buttons */
GtkWidget *rw_button;
GtkWidget *back_button;
GtkWidget *vars_button;
GtkWidget *forward_button;
GtkWidget *ff_button;

/* comments window */
GtkWidget *text;

#define MSG_MAX 256

/* state label */
GtkWidget *state;
#define STATE_DEF "Waiting to play"
gchar state_string[MSG_MAX];

/* statusbar */
GtkWidget *statusbar;
guint message_id = -1;
gchar message_string[MSG_MAX];

gchar *colstr[2] = {
	"Black",
	"White",
};

/* Forward declaration */
void report_move(gint col, gint i, gint j, gchar *name, gint nodenote, gint movenote);
void set_status_bar(gchar *msg);

/* info_callback
 * pops up info dialog box
 * FIXME - doesn't support multiple trees
 */
void info_callback(GtkWidget *w, gpointer data)
{

	struct game *g;
	g = (struct game *)data;
	info_dialog(g, g->col->tree);

}

/* settings callbacks
 * FIXME - what are these doing here?
 * They're getting use of 'go_board', that's what.
 * But it's bad.
 */
void settings_callback_display(GtkWidget *w, gpointer data)
{
	settings_dialog(go_board, (struct game *)data, GO_DISPLAY);
}
void settings_callback_players(GtkWidget *w, gpointer data)
{
	settings_dialog(go_board, (struct game *)data, GO_PLAYER);
}
void settings_callback_rules(GtkWidget *w, gpointer data)
{
	settings_dialog(go_board, (struct game *)data, GO_RULE);
}
void settings_callback_ai (GtkWidget *w, gpointer data)
{
	settings_dialog(go_board, (struct game *)data, GO_AI);
}

/* SGF navigation callbacks
 */

struct sgfnode *cur_node = NULL;
gint cur_var = 1;

void set_current(struct sgfnode *node, gint var)
{
	cur_node = node;
	cur_var = var;
}

/* set_vars_button
 * handles the variation toggle button
 * is only sensitive if there's a variation
 * otherwise shows 'O'
 */
void set_vars_button(gint var, gint num)
{

	GtkWidget *label;
	GString *varstr;

	varstr = g_string_new("O");

	if((var == 1) && (num == 1)) {
		gtk_widget_set_sensitive(vars_button, FALSE);
	} else {
		g_string_sprintf(varstr, "%d/%d", var, num);
		gtk_widget_set_sensitive(vars_button, TRUE);
	}
	label = (GTK_BIN(vars_button)->child);
	gtk_label_set_text(GTK_LABEL(label), g_strdup(varstr->str));


	g_string_free(varstr, TRUE);

}

/* set_resign_stop_button
 * if flag set, make it say 'Resign'
 * if unset, 'Stop'
 */
void set_resign_stop_button(gint flag)
{

	GtkWidget *label;

	label = (GTK_BIN(resign_button)->child);
	if(flag) {
		gtk_label_set_text(GTK_LABEL(label), "Resign");
	} else {
		gtk_label_set_text(GTK_LABEL(label), "Stop");
	}

}

void update_vars(struct sgfnode *node, gint var)
{

	gint num;

	sgf_get_vars(node, &var, &num);
	set_vars_button(var, num);

}

void vars_callback(GtkWidget *w, gpointer data)
{
	int x, y;
	struct game *g;
	g = (struct game *)data;
	if(g->mode == GOBAN_EDIT) {
		/* move to next var, count moves, then back to where we were */
		cur_node = sgf_nextvar(data, &x, &y, cur_node);
		g->num_moves = sgf_movecount(data, cur_node);
		report_move(!col_to_play(g->tomove), x + 1, g->boardsize - y,
			    g->nodename->str, g->nodenote, g->movenote);
		update_vars(cur_node, cur_var);
		sync_gui_board(data);
		update_state(data);
		set_text(g->comment->str);
	}

}


void rw_callback(GtkWidget *w, gpointer data)
{
	struct game *g;
	g = (struct game *)data;
	if(g->mode == GOBAN_EDIT) {
		cur_node = sgf_rw(data, cur_node);
		g->num_moves = sgf_movecount(data, cur_node);
		update_vars(cur_node, cur_var);
		set_status_bar(NULL);
		sync_gui_board(data);
		update_state(data);
		set_text(g->comment->str);
	}

}
void back_callback(GtkWidget *w, gpointer data)
{
	int x, y;
	struct game *g;
	g = (struct game *)data;
	if(g->mode == GOBAN_EDIT) {
		cur_node = sgf_back(data, &x, &y, cur_node);
		g->num_moves = sgf_movecount(data, cur_node);
		report_move(!col_to_play(g->tomove), x + 1, g->boardsize - y,
			    g->nodename->str, g->nodenote, g->movenote);
		update_vars(cur_node, cur_var);
		sync_gui_board(data);
		update_state(data);
		set_text(g->comment->str);
	}

}
void forward_callback(GtkWidget *w, gpointer data)
{
	int x, y;
	struct game *g;
	g = (struct game *)data;
	if(g->mode == GOBAN_EDIT) {
		cur_node = sgf_forward(data, &x, &y, cur_node);
		report_move(!col_to_play(g->tomove), x + 1, g->boardsize - y,
			    g->nodename->str, g->nodenote, g->movenote);
		update_vars(cur_node, cur_var);
		sync_gui_board(data);
		update_state(data);
		set_text(g->comment->str);
	}
}
void ff_callback(GtkWidget *w, gpointer data)
{
	struct game *g;
	g = (struct game *)data;
	if(g->mode == GOBAN_EDIT) {
		cur_node = sgf_ff(data, cur_node);
		cur_var = 1;
		if(g->have_result) {
			set_status_bar(g->score->str);
		} else {
			set_status_bar("End of file");
		}
		update_vars(cur_node, cur_var);
		sync_gui_board(data);
		update_state(data);
		set_text(g->comment->str);
	}
}

/* set_text
 * takes text, plonks it in the text widget
 * removing whatever was already there
 */
void set_text(gchar *msg)
{

//	gint p = 0;

	/* remove existing text */
	gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);

	/* insert new text */
//	gtk_editable_insert_text(GTK_EDITABLE(text), text, strlen(msg), &p);
	/* this is the gtk_text way, with NULLs where font and colour vals could go */
	gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, msg, -1);
	/* set position to beginning of text */
	gtk_editable_set_position(GTK_EDITABLE(text), 0);

}


/* set_user_strings
 * being Black, White and the info string
 */
void set_user_strings(struct game *g)
{

	gtk_frame_set_label(GTK_FRAME(uframe), g->info->str);
	gtk_entry_set_text(GTK_ENTRY(p1entry), g->pname[BLACK]->str);
	gtk_entry_set_text(GTK_ENTRY(p2entry), g->pname[WHITE]->str);

}

gint show_coords = TRUE;
gint coord_style = GTK_GO_BOARD_IGS_COORD;
gint hi_contrast = FALSE;

void set_soft_settings(gint sc, gint cs, gint hc)
{

	show_coords = sc;
	coord_style = cs;
	hi_contrast = hc;

	update_soft_settings();

}

/* update_soft_settings
 */
void update_soft_settings(void)
{
	/* update display settings */

	/* Need to turn show_coordinates off for set_coord_style to work properly */
	gtk_go_board_set_show_coordinates (GTK_GO_BOARD(go_board), FALSE);
	gtk_go_board_set_coord_style(GTK_GO_BOARD(go_board), coord_style);
	/* Now set actual show_coords value */
	gtk_go_board_set_show_coordinates (GTK_GO_BOARD(go_board), show_coords);
	gtk_go_board_set_high_contrast (GTK_GO_BOARD(go_board), hi_contrast);
}



/* set_status_bar
 * removes old message, adds new one
 */
void set_status_bar(gchar *msg)
{

	/* remove old message */
	if (message_id > 0)
		gtk_statusbar_remove (GTK_STATUSBAR (statusbar), 1, message_id);
	/* add new one */
	if(msg != NULL) {
		message_id = gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, msg);
	}

}

/* init_goban
 * just draw the goban widget
 * it goes on page of an invisible notebook
 * so it can be replaced in toto while program is running
 * this way, board size can be changed
 */
void init_goban(struct game *g)
{

	GtkWidget *frame;
	GtkWidget *label;

	/* An invisible frame with no label */
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	label = gtk_label_new("");

	if(gtk_notebook_get_current_page(GTK_NOTEBOOK(gframe)) == 0) {
		gtk_notebook_remove_page(GTK_NOTEBOOK(gframe), 0);

		gtk_widget_destroy(go_board);

	}

	go_board = gtk_go_board_new (g->boardsize, g->boardsize);

	gtk_signal_connect (GTK_OBJECT(go_board), "pressed",
			    GTK_SIGNAL_FUNC(pressed_callback), (gpointer)g);
	gtk_signal_connect (GTK_OBJECT(go_board), "enter",
			    GTK_SIGNAL_FUNC(enter_callback), (gpointer)g);
	gtk_signal_connect (GTK_OBJECT(go_board), "leave",
			    GTK_SIGNAL_FUNC(leave_callback), (gpointer)g);

	gtk_container_add(GTK_CONTAINER(frame), go_board);

	gtk_notebook_append_page (GTK_NOTEBOOK (gframe), frame, label);

	gtk_widget_show(go_board);
	gtk_widget_show(frame);
	gtk_widget_show(gframe);


}

/* create_goban
 * now badly misnamed
 * this creates most of the gui
 */
void create_goban (GtkWidget *vbox, struct game *g)
{
	GtkWidget *hbox, *vvbox, *bbox, *vvhbox;
	GtkWidget *frame, *framevbox;
	GtkWidget *label;
	GtkWidget *button;
	GtkWidget *scroll;

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

	/* vbox on left for go board and comments */
	vvbox = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), vvbox, TRUE, TRUE, 5);

	gframe = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(gframe), FALSE);
	gtk_box_pack_start(GTK_BOX (vvbox), gframe, TRUE, TRUE, 0);

	/* go board on the left */
	init_goban(g);

	/* Show coords on, IGS coordinate style, Hicontrast off */
	set_soft_settings(g->show_coords, g->coord_style, g->hi_contrast);

	/* comment window across bottom of go board */
	/* has scrollbar, hence vvhbox */
	vvhbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vvbox), vvhbox, FALSE, FALSE, 0);
	/* set an arbitrary height of 64 pixels for text box */
	gtk_widget_set_usize(vvhbox, 0, 64);
	/* ye text widget */
	text = gtk_text_new(NULL, NULL);
	/* wrap everything, but don't let us edit yet */
	gtk_text_set_editable(GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
	gtk_text_set_line_wrap(GTK_TEXT(text), TRUE);
	gtk_box_pack_start(GTK_BOX (vvhbox), text, TRUE, TRUE, 0);
	gtk_widget_show(text);
	/* add a scrollbar on the right */
	scroll = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
	gtk_box_pack_start(GTK_BOX(vvhbox), scroll, FALSE, FALSE, 0);
	gtk_widget_show(scroll);

	gtk_widget_show(vvhbox);
	gtk_widget_show(vvbox);

	/* vbox on the right of the go board */
	vvbox = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), vvbox, FALSE, TRUE, 5);
  
	/* frame for basic info */
	uframe = gtk_frame_new(g->info->str);
	gtk_box_pack_start(GTK_BOX(vvbox), uframe, FALSE, TRUE, 0);

	framevbox = gtk_vbox_new (TRUE, 2);
	gtk_container_add (GTK_CONTAINER (uframe), framevbox);

	label = gtk_label_new("Black:");
	gtk_box_pack_start(GTK_BOX(framevbox), label, FALSE, TRUE, 0);
	gtk_widget_show(label);

	p1entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(p1entry), FALSE);
	gtk_box_pack_start(GTK_BOX(framevbox), p1entry, FALSE, TRUE, 0);
	gtk_widget_show(p1entry);

	label = gtk_label_new("White:");
	gtk_box_pack_start(GTK_BOX(framevbox), label, FALSE, TRUE, 0);
	gtk_widget_show(label);

	p2entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(p2entry), FALSE);
	gtk_box_pack_start(GTK_BOX(framevbox), p2entry, FALSE, TRUE, 0);
	gtk_widget_show(p2entry);
	
	/* fill in player names */
	set_user_strings(g);

	gtk_widget_show (framevbox);
	gtk_widget_show (uframe);

	/* frame for play, pass, settings, quit buttons */
	frame = gtk_frame_new("Game");
	gtk_box_pack_start(GTK_BOX (vvbox), frame, FALSE, TRUE, 0);

	framevbox = gtk_vbox_new(TRUE, 2);
	gtk_container_add(GTK_CONTAINER(frame), framevbox);

	play_button = gtk_button_new_with_label("Play");
	gtk_signal_connect (GTK_OBJECT(play_button), "clicked",
			    GTK_SIGNAL_FUNC(play_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(framevbox), play_button, FALSE, TRUE, 10);
	gtk_widget_show(play_button);

	pass_button = gtk_button_new_with_label("Pass");
	gtk_signal_connect (GTK_OBJECT(pass_button), "clicked",
			    GTK_SIGNAL_FUNC(pass_callback), (gpointer)g);
	gtk_widget_set_sensitive(pass_button, FALSE);
	gtk_box_pack_start(GTK_BOX(framevbox), pass_button, FALSE, TRUE, 10);
	gtk_widget_show(pass_button);

	resign_button = gtk_button_new_with_label("Resign");
	gtk_signal_connect (GTK_OBJECT(resign_button), "clicked",
			    GTK_SIGNAL_FUNC(resign_callback), (gpointer)g);
	gtk_widget_set_sensitive(resign_button, FALSE);
	gtk_box_pack_start(GTK_BOX(framevbox), resign_button, FALSE, TRUE, 10);
	gtk_widget_show(resign_button);

	button = gtk_button_new_with_label("Info");
	gtk_signal_connect (GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(info_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(framevbox), button, FALSE, TRUE, 10);
	gtk_widget_show(button);

	gtk_widget_show(framevbox);
	gtk_widget_show(frame);

	/* frame for edit navigation */
	frame = gtk_frame_new("Controls");
	gtk_box_pack_start(GTK_BOX (vvbox), frame, FALSE, TRUE, 0);

	bbox = gtk_hbutton_box_new();
	gtk_button_box_set_child_size(GTK_BUTTON_BOX(bbox), 15, 15);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 2);
	gtk_container_add(GTK_CONTAINER(frame), bbox);

	rw_button = gtk_button_new_with_label("<<");
	gtk_signal_connect (GTK_OBJECT(rw_button), "clicked",
			    GTK_SIGNAL_FUNC(rw_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(bbox), rw_button, FALSE, FALSE, 0);
	gtk_widget_show(rw_button);

	back_button = gtk_button_new_with_label("<");
	gtk_signal_connect (GTK_OBJECT(back_button), "clicked",
			    GTK_SIGNAL_FUNC(back_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(bbox), back_button, FALSE, FALSE, 0);
	gtk_widget_show(back_button);	

	vars_button = gtk_button_new_with_label("O");
	gtk_signal_connect (GTK_OBJECT(vars_button), "clicked",
			    GTK_SIGNAL_FUNC(vars_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(bbox), vars_button, FALSE, FALSE, 0);
	gtk_widget_set_sensitive(vars_button, FALSE);
	gtk_widget_show(vars_button);	

	forward_button = gtk_button_new_with_label(">");
	gtk_signal_connect (GTK_OBJECT(forward_button), "clicked",
			    GTK_SIGNAL_FUNC(forward_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(bbox), forward_button, FALSE, FALSE, 0);
	gtk_widget_show(forward_button);

	ff_button = gtk_button_new_with_label(">>");
	gtk_signal_connect (GTK_OBJECT(ff_button), "clicked",
			    GTK_SIGNAL_FUNC(ff_callback), (gpointer)g);
	gtk_box_pack_start(GTK_BOX(bbox), ff_button, FALSE, FALSE, 0);
	gtk_widget_show(ff_button);

	gtk_widget_show(bbox);
	gtk_widget_show(frame);

	/* frame for game state */
	frame = gtk_frame_new("Status");
	gtk_box_pack_start(GTK_BOX (vvbox), frame, FALSE, TRUE, 0);

	framevbox = gtk_vbox_new(TRUE, 2);
	gtk_container_add(GTK_CONTAINER(frame), framevbox);

	state = gtk_label_new(STATE_DEF);
	gtk_box_pack_start(GTK_BOX(framevbox), state, FALSE, TRUE, 10);
	gtk_widget_show(state);

	gtk_widget_show(framevbox);
	gtk_widget_show(frame);

	/* show everything so far */
	gtk_widget_show (vvbox);
	gtk_widget_show (hbox);



	/* statusbar across bottom */
	statusbar = gtk_statusbar_new ();
	gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, TRUE, 0);
	gtk_widget_show (statusbar);
	
}

static guint pres_state = GTK_GO_BOARD_SOLID_BLACK;

static void toggle_pres_state(void)
{
	if(pres_state == GTK_GO_BOARD_SOLID_BLACK) {
		pres_state = GTK_GO_BOARD_SOLID_WHITE;
	} else {
		pres_state = GTK_GO_BOARD_SOLID_BLACK;
	}
}

/* update_state
 * set state label appropriate to mode and move number
 */
void update_state(struct game *g)
{

	gchar *fmt;

	switch(g->mode) {
	case GOBAN_PLAYW:
		fmt = "Move: %d\nClick board for next move\nB: %d, W: %d";
		g_snprintf(state_string, MSG_MAX, fmt, g->move, g->bcap, g->wcap);
		break;
	case GOBAN_PLAYH:
		if(col_to_play(g->tomove) == WHITE) {
			fmt = "Move: %d\nWhite to play\nB: %d, W: %d";
		} else {
			fmt = "Move: %d\nBlack to play\nB: %d, W: %d";
		}
		g_snprintf(state_string, MSG_MAX, fmt, g->move, g->bcap, g->wcap);
		break;	
	case GOBAN_PLAYC:
		if(col_to_play(g->tomove) == WHITE) {
			fmt = "Move: %d\nWhite thinking...\nB: %d, W: %d";
		} else {
			fmt = "Move: %d\nBlack thinking...\nB: %d, W: %d";
		}
		g_snprintf(state_string, MSG_MAX, fmt, g->move, g->bcap, g->wcap);
		break;
	case GOBAN_OVER:
		fmt = "Move: %d\nGame over - %s\nB: %d, W: %d";
		g_snprintf(state_string, MSG_MAX, fmt, g->move, g->score->str, g->bcap, g->wcap);
		break;
	case GOBAN_SCORE:
		fmt = "Calculating score...";
		g_snprintf(state_string, MSG_MAX, fmt);
		break;
	case GOBAN_EDIT:
		if(g->have_result && (g->move == g->num_moves)) {
			fmt = "Move: %d/%d\nResult - %s\nB: %d, W: %d";
			g_snprintf(state_string, MSG_MAX, fmt, g->move, g->num_moves, g->score->str, g->bcap, g->wcap);
			break;
		}
		if(col_to_play(g->tomove) == WHITE) {
			fmt = "Move: %d/%d\nWhite to play\nB: %d, W: %d";
		} else {
			fmt = "Move: %d/%d\nBlack to play\nB: %d, W: %d";
		}
		g_snprintf(state_string, MSG_MAX, fmt, g->move, g->num_moves, g->bcap, g->wcap);
		break;
	default:
		g_snprintf(state_string, MSG_MAX, STATE_DEF);
		break;

	}

	gtk_label_set_text(GTK_LABEL(state), state_string);

}
/* clear_gui_board
 * removes all stones from gui
 */
void clear_gui_board(gint size)
{
	gint i, j;

	for(i = 1; i < size+1; i++) {
		for(j = 1; j < size+1; j++) {
			gtk_go_board_set_stone(GTK_GO_BOARD(go_board), i, j,
					       GTK_GO_BOARD_EMPTY);
		}
	}

}

/* remove_gui_captured
 * scans board for dead stones
 * removes them from gui only
 */
void remove_gui_captured(struct game *g)
{

	gint i, j;

	for(i = 1; i < g->boardsize+1; i++) {
		for(j = 1; j < g->boardsize+1; j++) {
			if((g->board[i-1][j-1] == GOBAN_DW) ||
			   (g->board[i-1][j-1] == GOBAN_DB)) {
				gtk_go_board_set_stone(GTK_GO_BOARD(go_board), i, j,
						       GTK_GO_BOARD_EMPTY);
			}
		}
	}

}

/* sync_gui_board
 * scans the board in data structure
 * ensures gui board shows same
 * used for handicaps, initial positions, setup, etc
 */
void sync_gui_board(struct game *g)
{
	gint i, j;

	for(i = 1; i < g->boardsize+1; i++) {
		for(j = 1; j < g->boardsize+1; j++) {
			if((g->board[i-1][j-1] == GOBAN_W)) {
				if(gtk_go_board_get_stone (GTK_GO_BOARD(go_board), i, j) != GTK_GO_BOARD_SOLID_WHITE) {
					gtk_go_board_set_stone(GTK_GO_BOARD(go_board), i, j,
							       GTK_GO_BOARD_SOLID_WHITE);
				}
			} else if((g->board[i-1][j-1] == GOBAN_B)) {
				if(gtk_go_board_get_stone (GTK_GO_BOARD(go_board), i, j) != GTK_GO_BOARD_SOLID_BLACK) {
					gtk_go_board_set_stone(GTK_GO_BOARD(go_board), i, j,
							       GTK_GO_BOARD_SOLID_BLACK);
				}
			} else {
				if(gtk_go_board_get_stone (GTK_GO_BOARD(go_board), i, j) != GTK_GO_BOARD_EMPTY) {
					gtk_go_board_set_stone(GTK_GO_BOARD(go_board), i, j,
							       GTK_GO_BOARD_EMPTY);
				}
			}
		}
	}
}

/* prod_gtk
 * force a gtk update
 */
void prod_gtk(void)
{
	while(g_main_iteration(FALSE));
}

/* report_move
 * displays move in status bar
 * checks state of current coords style to do the Right Thing
 * Understands any of -1, 0 or 20 to mean pass
 */
//enum nodenotes { SN_NONE, SN_DM1, SN_DM2, SN_GB1, SN_GB2, SN_GW1, SN_GW2, SN_HO1, SN_HO2, SN_UC1, SN_UC2 };
//enum movenotes { SM_NONE, SM_BM1, SM_BM2, SM_DO, SM_IT, SM_TE1, SM_TE2 };
#define NN_NUM 11
#define MN_NUM 7

gchar *nn[NN_NUM] = {
	"",
	" - Even position",
	" - Main variation",
	" - Good for black",
	" - Very good for black",
	" - Good for white",
	" - Very good for white",
	" - Critical position",
	" - Very critical position",
	" - Unclear position",
	" - Very unclear position",
};
gchar *mn[MN_NUM] = {
	"",
	"? (Bad move)",
	"?? (Very bad move)",
	"?! (Doubtful move)",
	"!? (Interesting move)",
	"! (Tesuji)",
	"!! (Great tesuji)"
};

/* report_move
 * updates status bar
 * anything with -1 is no move
 * anything with 0 or 20 is a pass
 * anything else is a move
 */
void report_move(gint col, gint i, gint j, gchar *name, gint nodenote, gint movenote)
{

	gchar *colour;

	colour = colstr[col];

	/* sanity check nodenote and movenote */
	if(nodenote >= NN_NUM) {
		g_warning("report_move: bad nodenote %d\n", nodenote);
		nodenote = SN_NONE;
	}
	if(movenote >= MN_NUM) {
		g_warning("report_move: bad movenote %d\n", movenote);
		movenote = SM_NONE;
	}

	if((i == -1) || (j == -1)) {
		/* no move */
		g_snprintf(message_string, MSG_MAX, "%s %s", name, nn[nodenote]);
	} else if(((i == 0) || (j == 0)) ||
	   ((i == 20) || (j == 20))) {
		/* pass move */
		g_snprintf(message_string, MSG_MAX, "%s %s passes%s%s", name, colour, mn[movenote], nn[nodenote]);
	} else {

		/* normal move */
		if((i > 8) && (GTK_GO_BOARD(go_board)->coord_style == GTK_GO_BOARD_IGS_COORD)) {
			g_snprintf (message_string, MSG_MAX, "%s %s: %c, %d%s%s",
				    name, colour, 'A' + i, j, mn[movenote], nn[nodenote]);
		} else {
			g_snprintf (message_string, MSG_MAX, "%s %s: %c, %d%s%s",
				    name, colour, 'A' + i - 1, j, mn[movenote], nn[nodenote]);
		}
	}		

	set_status_bar(message_string);
	
}

/* FIXME - this is a crude hack */
gint aiai_stop = 0;

/* pressed_callback
 * handle presses on the game board
 * also handles all 'moves', which just seems wrong
 * still, it works for now
 * FIXME - loads of code happens twice here, should be factored out
 */
static void
pressed_callback (GtkGoBoard *board, gint x, gint y, gpointer data)
{
	guint stone;
	gint err;
	gint i, j;
	gint to_move;

	struct game *g;

	g = (struct game *)data;

	/* a click on the board will behave differently
	 * depending on what mode we're in
	 */
	switch(g->mode) {
	case GOBAN_PLAYW:
		/* only get here if computer/computer game */
		if(aiai_stop) {
			aiai_stop = 0;
			set_edit_buttons();
			cleanup_ai(g);
			g->mode = GOBAN_OVER;
		} else {
			g->mode = GOBAN_PLAYC;
			update_state(g);
		}
		pressed_callback(board, x, y, data);
		break;
	case GOBAN_PLAYC:
		/* nb - in this mode we ignore x and y */
		to_move = col_to_play(g->tomove);
		/* check we're not already waiting for one */
		if(g->t[to_move]->waiting)
			break;
		if(get_move(g->t[to_move], &i, &j, to_move)) {
			g_warning("pressed_callback: couldn't get move\n");
		}
		if((i == -1) && (j == -1)) {
			pass_callback(NULL, data);

		} else if ((i == -2) && (j == -2)) {
			resign_callback(NULL, data);

		} else {
			if((err = do_move(&g, i-1, j-1))) {
				g_warning("do_move: didn't like %d, %d\n", i, j);
			}
			/* show computers move */
			g->pass = 0;
			gtk_go_board_set_stone (board, i, j, pres_state);
			toggle_pres_state();
			remove_gui_captured(g);
			remove_captured(&g);

			to_move = col_to_play(g->tomove);
			
			/* write move down */
			g->num_moves++;
			cur_node = add_sgf_move(g->col->tree, i - 1, g->boardsize - j, !to_move);
			/* report move */
			report_move(!to_move, i, j, "", SN_NONE, SM_NONE);
//			g_snprintf (message_string, MSG_MAX, "%s: (%c, %d)",
//				    colstr[!to_move], 'A' + i - 1, j);
//			set_status_bar(message_string);

			/* set up for next player */
			if(g->p[to_move] != GOBAN_HUMAN) {
				g->mode = GOBAN_PLAYW;
				/* send move to computer */
				if(send_move(g->t[to_move], i, j, !to_move)) {
					g_warning("pressed_callback: error sending move\n");
				}
				/* get next computer move straight away */
				update_state(g);
				pressed_callback(board, x, y, data);
			} else {
				g->mode = GOBAN_PLAYH;
			}

		}
		break;

	case GOBAN_PLAYH:
		/* nb - x,y here are indexed on 1, but do_move wants index on 0 */
		if((err = do_move(&g, x-1, y-1))) {
			switch(err) {
			case GOBAN_ERR_RANGE:
				message_dialog("Oops", "Move out of range");
				break;
			case GOBAN_ERR_KO:
				message_dialog("Oops", "Illegal move: ko rule");
				break;
			case GOBAN_ERR_SUI:
				message_dialog("Oops", "Illegal move: suicide rule");
				break;
			case GOBAN_ERR_OCC:
				message_dialog("Oops", "Illegal move: already occupied");
				break;
			default:
				message_dialog("Oops", "Illegal move");
				break;
			}
			
		} else {
			
			stone = gtk_go_board_get_stone (board, x, y);
			
			if (stone == GTK_GO_BOARD_EMPTY 
			    || stone == GTK_GO_BOARD_TRANS_BLACK
			    || stone == GTK_GO_BOARD_TRANS_WHITE) {

				g->pass = 0;
				gtk_go_board_set_stone (board, x, y, pres_state);
				toggle_pres_state();
				remove_gui_captured(g);
				remove_captured(&g);

				to_move = col_to_play(g->tomove);

				/* write move down */
				g->num_moves++;
				cur_node = add_sgf_move(g->col->tree, x - 1, g->boardsize - y, !to_move);

				/* report move */
				report_move(!to_move, x, y, "", SN_NONE, SM_NONE);

//				g_snprintf (message_string, MSG_MAX, "%s: (%c, %d)",
//					    colstr[!to_move], 'A' + x - 1, y);
//				set_status_bar(message_string);

				/* set up for next player */
				if(g->p[to_move] != GOBAN_HUMAN) {
					g->mode = GOBAN_PLAYC;
					/* send move to computer */
//					g_print("Sending %d, %d\n", x, y);
					if(send_move(g->t[to_move], x, y, !to_move)) {
						g_warning("pressed_callback: error sending move\n");
					}
					/* call ourselves again to get computers move */
					update_state(g);
					pressed_callback(board, x, y, data);
				}

			} else {
				
				message_dialog("Oh dear", "GUI out of sync with data struct.");
				g_error("GUI out of sync with data, bailing\n");
			}
		}
		break;
	case GOBAN_OVER:

		/* Click on the board in game over state to reinitialise it */

		/* go back to 'edit' mode */
		g->mode = GOBAN_EDIT;
		update_state(g);
		/* reinitialise gui */
//		cur_var = 1;
//		cur_node = sgf_rw(data, g->col->tree->nodes);
//		cur_node = sgf_ff(data, g->col->tree->nodes);

		if(g->have_result) {
			set_status_bar(g->score->str);
		} else {
			set_status_bar("Game over");
		}
		update_vars(cur_node, cur_var);

		break;
	default:
		/* do nothing */
		break;

	}

	update_state(g);
  
}

/* set_play_buttons
 * enables/disables the right things for play mode
 */
void set_play_buttons(void)
{

	gtk_widget_set_sensitive(play_button, FALSE);
	gtk_widget_set_sensitive(pass_button, TRUE);
	gtk_widget_set_sensitive(resign_button, TRUE);

	gtk_widget_set_sensitive(rw_button, FALSE);
	gtk_widget_set_sensitive(back_button, FALSE);
	gtk_widget_set_sensitive(forward_button, FALSE);
	gtk_widget_set_sensitive(ff_button, FALSE);
	gtk_widget_set_sensitive(vars_button, FALSE);

}

/* set_edit_buttons
 * enables/disables the right things for edit mode
 */
void set_edit_buttons(void)
{

	gtk_widget_set_sensitive(play_button, TRUE);
	gtk_widget_set_sensitive(pass_button, FALSE);
	gtk_widget_set_sensitive(resign_button, FALSE);

	gtk_widget_set_sensitive(rw_button, TRUE);
	gtk_widget_set_sensitive(back_button, TRUE);
	gtk_widget_set_sensitive(forward_button, TRUE);
	gtk_widget_set_sensitive(ff_button, TRUE);
	gtk_widget_set_sensitive(vars_button, FALSE);


}
/* play_callback
 * puts game in play mode
 */
static void play_callback(GtkWidget *button, gpointer data)
{

	gint go;
	struct game *g;

	g = (struct game *)data;

	/* check we're not waiting for a computer to report score */
	go = col_to_play(g->tomove);
	if(g->p[go] == GOBAN_AI) {
		if(g->t[go] && g->t[go]->waiting)
			return;
	}

	/* reinitialise data */
	g->mode = GOBAN_PLAYH;

	if(g->move == 0)
		init_game(&g, 0);

	/* initialise computer players */
	if(g->p[WHITE] != GOBAN_HUMAN) {
		if(init_ai(g, WHITE, &prod_gtk)) {
			g->mode = GOBAN_ERROR;
			cleanup_ai(g);
			return;
		}
		set_sgf_prop(g->col->tree, SP_PW, g->pname[WHITE]->str);
		if((!g->move && g->handicap > 0) ||
		   (g->tomove == GOBAN_W))
			g->mode = GOBAN_PLAYC;

	}
	if(g->p[BLACK] != GOBAN_HUMAN) {
		if(init_ai(g, BLACK, &prod_gtk)) {
			g->mode = GOBAN_ERROR;
			cleanup_ai(g);
			return;
		}
		set_sgf_prop(g->col->tree, SP_PB, g->pname[BLACK]->str);
		if((!g->move && g->handicap == 0) ||
		   (g->tomove == GOBAN_B))
			g->mode = GOBAN_PLAYC;

	}

	/* change message */
	update_state(g);

	/* reinitialise gui */
	set_text("");
	set_user_strings(g);
	clear_gui_board(g->boardsize);
	pres_state = ((g->tomove == GOBAN_B) ? GTK_GO_BOARD_SOLID_BLACK : GTK_GO_BOARD_SOLID_WHITE);
	set_play_buttons();

	if((g->p[BLACK] == GOBAN_AI) && (g->p[WHITE] == GOBAN_AI)) {
		/* disable pass button, make resign button say Stop */
		gtk_widget_set_sensitive(pass_button, FALSE);
		gtk_widget_set_sensitive(resign_button, TRUE);
		set_resign_stop_button(0);
		aiai_stop = 0;
	} else {
		set_resign_stop_button(1);
	}

	/* Ensure board is synced */
	sync_gui_board(g);

	/* If we're starting midway through an SGF file, kill unmade moves */
	if(g->num_moves && (g->move < g->num_moves)) {
		cull_sgf_moves(g);
		g->num_moves = g->move;
	}

	/* if a computer is first to move, prod them */
	if(g->mode == GOBAN_PLAYC)
		pressed_callback(GTK_GO_BOARD(go_board), 0, 0, data);

}

/* game_over
 * reset gui for game over
 * calculate score string if we have an ai to do this for us
 * close down and cleanup any ai
 */
void game_over(struct game *g, gint resignation)
{
	//	gint i;
	gint move;
	gint have_score = 0;

	/* Default to being about to calculate score */
	g->mode = GOBAN_SCORE;

	set_edit_buttons();

	move = col_to_play(g->tomove);

	if(resignation) {
		/* No need to calculate score */
		g->mode = GOBAN_OVER;
		if(!col_to_play(g->tomove) == WHITE) {
			g->score = g_string_assign(g->score, "W+R");
		} else {
			g->score = g_string_assign(g->score, "B+R");
		}
		have_score = 1;
	} else {

		if(g->t[move]) {
			/* Say we're about to calculate score */
			update_state(g);
			/* Calculate score */
			send_command(g->t[move], GT_FINAL_SCORE);
			g->score = g_string_assign(g->score, g_strdup(g->t[move]->msg));
			have_score = 1;
		}
		
		if(g->t[!move]) {
			if(have_score == 0) {
				/* Say we're about to calculate score */
				update_state(g);
				/* Do it */
				send_command(g->t[!move], GT_FINAL_SCORE);
				g->score = g_string_assign(g->score, g_strdup(g->t[!move]->msg));
				have_score = 1;
			}
		}
		
		if(!have_score) {
			g->score = g_string_assign(g->score, "not scored");
		}
		/* g->mode must be GOBAN_OVER now whatever */
		g->mode = GOBAN_OVER;

	}

	/* Record result in SGF */
	set_sgf_prop(g->col->tree, SP_RE, g->score->str);
	g->have_result = 1;

	update_state(g);
	cleanup_ai(g);
	pressed_callback(GTK_GO_BOARD(go_board), 0, 0, (gpointer)g);

}

/* pass_callback
 * FIXME - pass rule is not always same, should not be hardwired like this
 */
static void pass_callback(GtkWidget *button, gpointer data)
{

	gint go;

	struct game *g;
	g = (struct game *)data;

	go = col_to_play(g->tomove);

	/* check we're not waiting for a computer move */
	if(g->p[go] == GOBAN_AI) {
		if(g->t[go]->waiting)
			return;
	}

 	toggle_pres_state();
	g->move++;
	toggle_tomove(g);
	g->num_moves++;
	g->pass++;

	add_sgf_move(g->col->tree, -1, -1, go);

	if(g->pass == 2) {
		set_status_bar("Game over.");
		game_over(g, 0);
		return;
	}

	if(go == WHITE) {

		if(g->p[go] !=GOBAN_HUMAN)
			report_move(WHITE, 0, 0, "", SN_NONE, SM_NONE);

	} else if(go == BLACK) {

		if(g->p[go] !=GOBAN_HUMAN)
			report_move(BLACK, 0, 0, "", SN_NONE, SM_NONE);

	}

        /* set up for next player */
	if(g->p[!go] != GOBAN_HUMAN) {
		g->mode = GOBAN_PLAYC;
		/* send move to computer */
//		g_print("Sending %d, %d\n", -1, -1);
		if(send_move(g->t[!go], -1, -1, go)) {
			g_warning("pass_callback: error sending pass\n");
		}

		/* call ourselves again to get computers move */
		if(g->pass < 2) {
			update_state(g);
			pressed_callback(GTK_GO_BOARD(go_board), -1, -1, data);
		}



	} else {
		/* computer may have just passed
		 * ensure human gets next go */
		g->mode = GOBAN_PLAYH;
	}

	update_state(g);

}

/* resign_callback
 * QUERY - resignation not sent to computer players. Does this matter?
 */
static void resign_callback(GtkWidget *button, gpointer data)
{

	gint go;

	struct game *g;
	g = (struct game *)data;

	/* handle aiai games differently */
	if((g->p[BLACK] == GOBAN_AI) && (g->p[WHITE] == GOBAN_AI)) {
		if(g->t[BLACK]->waiting || g->t[WHITE]->waiting) { 
			/* not real resignation - stop button pressed */
			aiai_stop = 1;
			return;
		}
	}

	go = col_to_play(g->tomove);

	/* check we're not waiting for a computer move */
	if(g->p[go] == GOBAN_AI) {
		if(g->t[go]->waiting)
			return;
	}

	if(go == WHITE) {
		set_status_bar("White resigns");
	} else if(go == BLACK) {
		set_status_bar("Black resigns");
	} 

	game_over(g, 1);

}


/* enter_callback
 * originally from testgoboard.c
 * now messed with
 * adds transparent stones as mouse moves around board
 */
static void
enter_callback (GtkGoBoard *board, gint x, gint y, gpointer data)
{
	guint cell;

	if(((struct game *)data)->mode == GOBAN_PLAYH) {

		cell = gtk_go_board_get_stone (board, x, y);
		
		if (cell == GTK_GO_BOARD_EMPTY) {
			if (pres_state == GTK_GO_BOARD_SOLID_BLACK)
				gtk_go_board_set_stone (board, x, y,
							GTK_GO_BOARD_TRANS_BLACK);
			else
				gtk_go_board_set_stone (board, x, y,
							GTK_GO_BOARD_TRANS_WHITE);
		}
	}
}

/* leave_callback
 * from testgoboard.c
 * blanks stones as mouse moves around board
 */
static void
leave_callback (GtkGoBoard *board, gint x, gint y, gpointer data)
{
	guint cell;

	if(((struct game *)data)->mode == GOBAN_PLAYH) {

		cell = gtk_go_board_get_stone (board, x, y);
		
		if (cell == GTK_GO_BOARD_TRANS_BLACK
		    || cell == GTK_GO_BOARD_TRANS_WHITE) {
			gtk_go_board_set_stone (board, x, y, GTK_GO_BOARD_EMPTY);
		}
		
	}
	
}



