/*
 * gui_menu.c
 */

#include "goben.h"
#include "gui_misc.h"
#include "gui_actions.h"
#include "game.h"
#include "gui_goboard.h"

/*
 * create_main_menu
 */
void create_main_menu(GtkWidget *window, GtkWidget *container, struct game *g)
{
        GtkWidget *menubar;
        GtkWidget *menu;
        GtkWidget *menuitem;
        
        /* Create accel table */
        accel_group = gtk_accel_group_new();
        gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
        
        /* Create tooltips */
        tooltips = gtk_tooltips_new();
        
        /* Menu bar */
        menubar = gtk_menu_bar_new();
        gtk_box_pack_start(GTK_BOX(container), menubar, FALSE, TRUE, 0);
        gtk_widget_show(menubar);

        /* File menu */
        menu = create_bar_submenu(menubar, "File");
        menuitem = create_menu_item(window, menu, "New Game", "^N",
				    "Clear board and start new game",
				    GTK_SIGNAL_FUNC(action_new), (gpointer)g);
        menuitem = create_menu_item(window, menu, "Load File...", "^L",
				    "Open an existing file",
				    GTK_SIGNAL_FUNC(action_open), (gpointer)g);
        menuitem = create_menu_item(window, menu, "Save File", "^S",
				    "Save current file",
				    GTK_SIGNAL_FUNC(action_save), (gpointer)g);
        menuitem = create_menu_item(window, menu, "Save File As...", "",
				    "Save current file with new name",
				    GTK_SIGNAL_FUNC(action_save_as), (gpointer)g);
	menuitem = create_menu_item(window, menu, NULL, NULL, NULL, NULL, NULL);
        menuitem = create_menu_item(window, menu, "Quit", "",
				    "cya",
				    GTK_SIGNAL_FUNC(action_quit), "quit");
        /* Settings menu */
        menu = create_bar_submenu(menubar, "Settings");
        menuitem = create_menu_item(window, menu, "Display Settings...", "",
				    "Display Settings", 
				    GTK_SIGNAL_FUNC(settings_callback_display), (gpointer)g);
        menuitem = create_menu_item(window, menu, "Rule Settings...", "",
				    "Rule Settings", 
				    GTK_SIGNAL_FUNC(settings_callback_rules), (gpointer)g);
        menuitem = create_menu_item(window, menu, "Player Settings...", "",
				    "Player Settings", 
				    GTK_SIGNAL_FUNC(settings_callback_players), (gpointer)g);
	menuitem = create_menu_item(window, menu, "AI Settings...", "",
				    "AI Settings", 
				    GTK_SIGNAL_FUNC(settings_callback_ai), (gpointer)g);
        /* Help menu */
        menu = create_bar_submenu(menubar, "Help");
        menuitem = create_menu_item(window, menu, "About", "",
				    "About",
				    GTK_SIGNAL_FUNC(action_about), "about");

}
