/*
 * gui_misc.c
 */

#include <gtk/gtk.h>
#include "gui_misc.h"

GtkWidget *create_widget_from_xpm(GtkWidget *window, gchar **xpm_data)
{

        GdkBitmap *mask;
        GdkPixmap *pixmap_data;
        GtkWidget *pixmap_widget;
        GdkColormap *mycolormap;
        
        mycolormap = gtk_widget_get_colormap(GTK_WIDGET(window));
        
        /* make gdk pixmap */
        pixmap_data = gdk_pixmap_colormap_create_from_xpm_d (NULL,
                                                             mycolormap,
                                                             &mask,
                                                             NULL,
                                                             (gchar **) xpm_data);
        
        
        /* convert pixmap to pixmap widget */
        pixmap_widget = gtk_pixmap_new(pixmap_data, mask);
        
        /* make widget visible */
        gtk_widget_show(pixmap_widget);
        
        /* return it */
        return (pixmap_widget);

}

/* create_menu_item
 * Harlow p 78
 * 
 * Creates menu item, adds to menu, returns item
 *
 * window - top level window for accels/tooltips
 * menu - container menu
 * name - name of menu - NULL for seperator
 * accel - Acceleration string - "^C" for Ctrl-C
 * tip - Tooltip
 * func - Callback function
 * data - Callback function data
 *
 */
GtkWidget *create_menu_item(GtkWidget *window,
			    GtkWidget *menu,
			    char *name,
			    char *accel,
			    char *tip,
			    GtkSignalFunc func,
			    gpointer data)
{
        GtkWidget *menuitem;
        
        /* If there's a name, create item and add signal handler */
        if(name && strlen(name)) {
                menuitem = gtk_menu_item_new_with_label(name);
                gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
                                   GTK_SIGNAL_FUNC(func), data);
        } else {
                /* Create separator */
                menuitem = gtk_menu_item_new();
        }
        
        /* Add menu item to menu and show it */
        gtk_menu_append(GTK_MENU(menu), menuitem);
        gtk_widget_show(menuitem);
        
        if(accel_group == NULL) {
                accel_group = gtk_accel_group_new();
                gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
        }
        
        /* If there was an accelerator */
        if (accel && accel[0] == '^') {
                gtk_widget_add_accelerator(menuitem,
                                           "activate",
                                           accel_group,
                                           accel[1],
                                           GDK_CONTROL_MASK,
                                           GTK_ACCEL_VISIBLE);
        }

        /* If there was a tooltip */
        if(tip && strlen(tip)) {
                gtk_tooltips_set_tip(tooltips, menuitem, tip, NULL);
        }
        
        return(menuitem);
}

/* create_bar_submenu
 * Harlow p74
 *
 * menu - higher level menu
 * name - submenu item name to be added
 *
 * Right justifies any menu called Help
 */
GtkWidget *create_bar_submenu(GtkWidget *menu, char *name)
{
        GtkWidget *menuitem;
        GtkWidget *submenu;
        
        /* Create menu */
        menuitem = gtk_menu_item_new_with_label(name);
        
        /* Add to bar */
        gtk_menu_bar_append(GTK_MENU_BAR(menu), menuitem);
        
        /* Follow tradition */
        if (g_strcasecmp(name, "Help") == 0) {
                gtk_menu_item_right_justify(GTK_MENU_ITEM(menuitem));
        }
        
        gtk_widget_show(menuitem);
        
        /* Get menu and attach to item */
        submenu = gtk_menu_new();
        gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
        
        return (submenu);
        
}
