/*
 * gui_settings.c
 */

#include <stdlib.h>

#include "goben.h"
#include "gui_settings.h"
#include "gui_actions.h"
#include "gtkgoboard.h"
#include "gui_goboard.h"
#include "aihandle.h"
#include "cfg.h"

/*****************/
/* Settings Data */
/*****************/

struct settings *temp_settings = NULL;

/* free_temp_settings
 * lose temporary struct and temporary strings
 */
static void free_temp_settings(void)
{

	gint i;

	if(temp_settings) {

		for(i = 0; i < 2; i++) {
			if(temp_settings->pname[i])
				g_string_free(temp_settings->pname[i], TRUE);
			if(temp_settings->path[i])
				g_string_free(temp_settings->path[i], TRUE);
			if(temp_settings->opts[i])
				g_string_free(temp_settings->opts[i], TRUE);
		}

		if(temp_settings->info)
			g_string_free(temp_settings->info, TRUE);
		
		free(temp_settings);

	}

}

/* make_temp_options
 * initialise a temporary struct to hold copy of all options
 */
static void make_temp_settings(GtkWidget *go_board, struct game *g)
{

	gint i;
	static gint boardsize;
	static gint handicap;
	static gdouble komi;

	temp_settings = malloc(sizeof(struct settings));

	temp_settings->coord_style = GTK_GO_BOARD (go_board)->coord_style;
	temp_settings->show_coords = GTK_GO_BOARD (go_board)->show_coordinates;
	temp_settings->hi_contrast = GTK_GO_BOARD (go_board)->high_contrast;

	for(i = 0; i < 2; i++) {
		temp_settings->ptype[i] = g->p[i];
		temp_settings->pname[i] = g_string_new(g->pname[i]->str);
		temp_settings->path[i] = g_string_new(g->path[i]->str);
		temp_settings->opts[i] = g_string_new(g->opts[i]->str);
	}

	temp_settings->info = g_string_new(g->info->str);

	/* FIXME - what is this pointer stuff for? I have no idea why I did this */
	handicap = g->handicap;
	boardsize = g->boardsize;
	komi = g->komi;
	temp_settings->handicap = &handicap;
	temp_settings->boardsize = &boardsize;
	temp_settings->komi = &komi;

	temp_settings->go_board = go_board;
	temp_settings->g = g;

	temp_settings->playflag = 0;
	if((temp_settings->g->mode == GOBAN_PLAYH) ||
	   (temp_settings->g->mode == GOBAN_PLAYC) ||
	   (temp_settings->g->mode == GOBAN_PLAYW)) {
		temp_settings->playflag = 1;
	}


}

/**********************/
/* Settings Callbacks */
/**********************/

/* option_toggled_callback
 * from testgoboard.c
 */
static void option_toggled_callback (GtkCheckButton *check, gpointer data)
{
  switch ((int)data)
    {
    case COORD_STYLE:
	    if(GTK_TOGGLE_BUTTON(check)->active) {
		    temp_settings->coord_style = GTK_GO_BOARD_IGS_COORD;
	    } else {
		    temp_settings->coord_style = GTK_GO_BOARD_NORMAL_COORD;
	    }
	    break;
    case HIGH_CONTRAST:
	    temp_settings->hi_contrast = GTK_TOGGLE_BUTTON(check)->active;
	    break;
    case SHOW_COORDINATES:
	    temp_settings->show_coords = GTK_TOGGLE_BUTTON(check)->active;
	    break;
    case BLACK_PTYPE:
	    temp_settings->ptype[BLACK] = GTK_TOGGLE_BUTTON(check)->active;
	    break;
    case WHITE_PTYPE:
	    temp_settings->ptype[WHITE] = GTK_TOGGLE_BUTTON(check)->active;
	    break;
    default:
	    break;
    }
}

/* change_string_callback
 * generic callback for string options
 * updates the GString pointed to by data
 */
static void change_string_callback(GtkWidget *widget, gpointer data)
{
	
	char *tmpstr = gtk_entry_get_text (GTK_ENTRY (widget));
	
	g_string_assign((GString *) data, tmpstr);
	
  
}

/* change_int_callback
 * generic callback for int options
 * updates int pointed to by data
 */
static void change_int_callback(GtkWidget *widget, gpointer data)
{
	gint *tmp;
	tmp = (gint *)data;
	*tmp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

/* change_float_callback
 * generic callback for float options
 * updates float pointed to by data
 */
static void change_float_callback(GtkWidget *widget, gpointer data)
{
	gdouble *tmp;
	tmp = (gdouble *)data;
	*tmp = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(widget));
}


/*
 * save_settings
 * writes temp_settings to game struct
 */
static void save_settings(void)
{

	gint i;

	if(temp_settings->playflag == 0) {

		/* update pairs */
		for (i = 0; i < 2; i++) {
			temp_settings->g->p[i] = temp_settings->ptype[i];
			g_string_assign(temp_settings->g->pname[i], g_strdup(temp_settings->pname[i]->str));
			g_string_assign(temp_settings->g->path[i], g_strdup(temp_settings->path[i]->str));
			g_string_assign(temp_settings->g->opts[i], g_strdup(temp_settings->opts[i]->str));
			
			/* If computer player selected, check it will work */
			if(temp_settings->g->p[i]) {
				if (init_ai(temp_settings->g, i, &prod_gtk)) {
				/* returned error, it won't work */
					temp_settings->g->p[i] = FALSE;
				}
				cleanup_ai(temp_settings->g);
			}
			
		}
		
		g_string_assign(temp_settings->g->info, g_strdup(temp_settings->info->str));
		
		/* update digits */
		temp_settings->g->komi = *(temp_settings->komi);
		
		/* update gui */
		if(temp_settings->g->boardsize != *(temp_settings->boardsize)) {
			temp_settings->g->boardsize = *(temp_settings->boardsize);
			init_goban(temp_settings->g);
			
		}
		
		/* allow any handicap except 1, which is meaningless */
		if(*(temp_settings->handicap) != 1)
			temp_settings->g->handicap = *(temp_settings->handicap);
		
		/* cap handicap at 4 on even sized boards though */
		if((temp_settings->g->handicap > 4) &&
		   (!(temp_settings->g->boardsize & 1))) {
			temp_settings->g->handicap = 4;
		}
		
		/* ban all handicaps on boards smaller than 7x7 */
		if(temp_settings->g->boardsize < 7) {
			temp_settings->g->handicap = 0;
		}
		
		set_user_strings(temp_settings->g);

	}

	temp_settings->g->show_coords = temp_settings->show_coords;
	temp_settings->g->coord_style = temp_settings->coord_style;
	temp_settings->g->hi_contrast = temp_settings->hi_contrast;
	set_soft_settings(temp_settings->show_coords, temp_settings->coord_style,
			  temp_settings->hi_contrast);

}


/* options_save
 * sets new settings then writes new cfg file
 */
static void options_save(GtkWidget *w, gpointer data)
{

	save_settings();
	if(save_cfg(temp_settings->g)) {
		temp_settings->g->need_save = 0;
	}
	gtk_widget_destroy (GTK_WIDGET (data));

}

/* options_ok
 * sets new settings. doesn't save cfg file
 */
static void options_ok(GtkWidget *w, gpointer data)
{

	save_settings();
	temp_settings->g->need_save = 1;
	gtk_widget_destroy (GTK_WIDGET (data));

}

/* destroy
 * releases focus grabbed by file selection widget
 * called on a "destroy" event, so no need to call gtk_widget_destroy
 * gtk_widget_destroy calls this...
 * Harlow p157
 */
static void destroy(GtkWidget *widget, gpointer data)
{

	/* free the temp_settings */
	free_temp_settings();
	
	/* remove focus */
	gtk_grab_remove (widget);
	
}

/* kill_window
 * window to kill is data
 */
static void kill_window(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy (GTK_WIDGET (data));
}

/*************************/
/* Settings Dialog Pages */
/*************************/

/* make_ai_settings
 * makes the ai settings page of the notebook
 */
GtkWidget *make_ai_settings(GtkWidget *widget)
{
	GtkWidget *label;
	GtkWidget *frame, *sframe;
	GtkWidget *vbox, *svbox;
	
	GtkWidget *entry;

	label = gtk_label_new ("AI");
	frame = gtk_frame_new ("AI Settings");
	gtk_notebook_append_page (GTK_NOTEBOOK (widget), frame, label);

	vbox = gtk_vbox_new (TRUE, 20);
	gtk_container_add (GTK_CONTAINER (frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);

	sframe = gtk_frame_new("Black:");
	gtk_box_pack_start (GTK_BOX (vbox), sframe, TRUE, TRUE, 0);
	svbox = gtk_vbox_new (TRUE, 20);
	gtk_container_add (GTK_CONTAINER (sframe), svbox);
//	gtk_container_set_border_width(GTK_CONTAINER (svbox), 10);

	label = gtk_label_new("Path:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (svbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->path[BLACK]->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->path[BLACK]);
	gtk_box_pack_start (GTK_BOX (svbox), entry, TRUE, TRUE, 0);

	label = gtk_label_new("Options:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (svbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->opts[BLACK]->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->opts[BLACK]);
	gtk_box_pack_start (GTK_BOX (svbox), entry, TRUE, TRUE, 0);

	sframe = gtk_frame_new("White:");
	gtk_box_pack_start (GTK_BOX (vbox), sframe, FALSE, TRUE, 0);
	svbox = gtk_vbox_new (TRUE, 20);
	gtk_container_add (GTK_CONTAINER (sframe), svbox);
//	gtk_container_set_border_width(GTK_CONTAINER (svbox), 10);

	label = gtk_label_new("Path:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (svbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->path[WHITE]->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->path[WHITE]);
	gtk_box_pack_start (GTK_BOX (svbox), entry, TRUE, TRUE, 0);

	label = gtk_label_new("Options:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (svbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->opts[WHITE]->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->opts[WHITE]);
	gtk_box_pack_start (GTK_BOX (svbox), entry, TRUE, TRUE, 0);

	/*
	 * Later:
	 * - GMP/GTP mode selection (when GMP implemented)
	 */

	return frame;

}

/* make_player_settings
 * makes the player settings page of the notebook
 */
GtkWidget *make_player_settings(GtkWidget *widget)
{

	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *check;
	GtkWidget *entry;

	label = gtk_label_new ("Player");
	frame = gtk_frame_new ("Player Settings");
	gtk_notebook_append_page (GTK_NOTEBOOK (widget), frame, label);
	
	vbox = gtk_vbox_new (TRUE, 20);
	gtk_container_add (GTK_CONTAINER (frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);

	/* add stuff to vbox here */

	/* Options to implement
	 * Now:
	 * - Player 1 name
	 * - Player 2 name
	 * - Info
	 */

	label = gtk_label_new("Black:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->pname[BLACK]->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->pname[BLACK]);
	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);

	check = gtk_check_button_new_with_label ("Black Is Computer?");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (check),
				     temp_settings->ptype[BLACK]);
	gtk_signal_connect (GTK_OBJECT (check), "toggled",
			    GTK_SIGNAL_FUNC (option_toggled_callback),
			    (gpointer)BLACK_PTYPE);
	gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, TRUE, 0);

	if((temp_settings->g->mode == GOBAN_PLAYH) ||
	   (temp_settings->g->mode == GOBAN_PLAYC))
		gtk_widget_set_sensitive(check, FALSE);


	label = gtk_label_new("White:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->pname[WHITE]->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->pname[WHITE]);
	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);

	check = gtk_check_button_new_with_label ("White Is Computer?");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (check),
				     temp_settings->ptype[WHITE]);
	gtk_signal_connect (GTK_OBJECT (check), "toggled",
			    GTK_SIGNAL_FUNC (option_toggled_callback),
			    (gpointer)WHITE_PTYPE);
	gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, TRUE, 0);

	if((temp_settings->g->mode == GOBAN_PLAYH) ||
	   (temp_settings->g->mode == GOBAN_PLAYC))
		gtk_widget_set_sensitive(check, FALSE);


	label = gtk_label_new("Info:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY (entry), 80);
	gtk_entry_set_text (GTK_ENTRY (entry), temp_settings->info->str);
	gtk_signal_connect (GTK_OBJECT (entry), "changed",
			    GTK_SIGNAL_FUNC (change_string_callback), (gpointer)temp_settings->info);
	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);

	return frame;

}

/* make_rule_settings
 * makes the rules settings page of the notebook
 */
GtkWidget *make_rule_settings(GtkWidget *widget)
{

	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *vbox;

	GtkAdjustment *spinner_adj;	
	GtkWidget *spinner;

	label = gtk_label_new ("Rules");
	frame = gtk_frame_new ("Rules Settings");
	gtk_notebook_append_page (GTK_NOTEBOOK (widget), frame, label);
	
	vbox = gtk_vbox_new (TRUE, 20);
	gtk_container_add (GTK_CONTAINER (frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);

	/* add stuff to vbox here */

	/* Implemented:
	 * - Boardsize
	 * - Handicap
	 * - Komi
	 */
	label = gtk_label_new("Boardsize:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

	/* set spin button adjustment, values for boardsize - skip by two
	 * to force people to explicitly type in an even number (and so lose
	 * the center point)
	 */
	spinner_adj = (GtkAdjustment *) gtk_adjustment_new(temp_settings->g->boardsize,
							   BOARD_MIN, BOARD_MAX, 2.0, 1.0, 1.0);
	/* creates the spinner, with no decimal places */
	spinner = gtk_spin_button_new (spinner_adj, 1.0, 0);
	gtk_signal_connect (GTK_OBJECT (spinner), "changed",
			    GTK_SIGNAL_FUNC (change_int_callback), (gpointer)temp_settings->boardsize);
	gtk_box_pack_start (GTK_BOX (vbox), spinner, FALSE, FALSE, 0);

	if((temp_settings->g->mode == GOBAN_PLAYH) ||
	   (temp_settings->g->mode == GOBAN_PLAYC))
		gtk_widget_set_sensitive(spinner, FALSE);

	label = gtk_label_new("Handicap:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	spinner_adj = (GtkAdjustment *) gtk_adjustment_new(temp_settings->g->handicap,
							   HANDICAP_MIN, HANDICAP_MAX, 1.0, 1.0, 1.0);
	/* creates the spinner, with no decimal places */
	spinner = gtk_spin_button_new (spinner_adj, 1.0, 0);
	gtk_signal_connect (GTK_OBJECT (spinner), "changed",
			    GTK_SIGNAL_FUNC (change_int_callback), (gpointer)temp_settings->handicap);
	gtk_box_pack_start (GTK_BOX (vbox), spinner, FALSE, FALSE, 0);

	if((temp_settings->g->mode == GOBAN_PLAYH) ||
	   (temp_settings->g->mode == GOBAN_PLAYC))
		gtk_widget_set_sensitive(spinner, FALSE);

	label = gtk_label_new("Komi:");
	gtk_misc_set_alignment(GTK_MISC (label), 0, 1.0);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

	/* set spin button adjustment, values for boardsize */
	spinner_adj = (GtkAdjustment *) gtk_adjustment_new(temp_settings->g->komi,
							   KOMI_MIN, KOMI_MAX, 0.25, 1.0, 1.0);
	/* creates the spinner, with no decimal places */
	spinner = gtk_spin_button_new (spinner_adj, 0.25, 2);
	gtk_signal_connect (GTK_OBJECT (spinner), "changed",
			    GTK_SIGNAL_FUNC (change_float_callback), (gpointer)temp_settings->komi);
	gtk_box_pack_start (GTK_BOX (vbox), spinner, FALSE, FALSE, 0);

	if((temp_settings->g->mode == GOBAN_PLAYH) ||
	   (temp_settings->g->mode == GOBAN_PLAYC))
		gtk_widget_set_sensitive(spinner, FALSE);
	/*
	 * Later:
	 * - Free handicap?
	 * - Allow suicide?
	 * - Ko rule type
	 * - Time, byo yomi (?) settings
	 * - Ruleset
	 * Ruleset can then be implemented as canned sets of settings for the other settings.
	 * Everything on this page should be disabled if game state is GOBAN_PLAY
	 */


	return frame;
	

}


/* make_display_settings
 * makes the display settings page of the notebook
 */
GtkWidget *make_display_settings(GtkWidget *widget)
{

	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *vbox;
	
	GtkWidget *check;

	label = gtk_label_new ("Display");
	frame = gtk_frame_new ("Display Settings");
	gtk_notebook_append_page (GTK_NOTEBOOK (widget), frame, label);
	
	vbox = gtk_vbox_new (TRUE, 20);
	gtk_container_add (GTK_CONTAINER (frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);
	
	/* options go here */

	check = gtk_check_button_new_with_label ("IGS Coordinates Style (omits 'i')");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (check),
				     ((GTK_GO_BOARD (temp_settings->go_board)->coord_style == GTK_GO_BOARD_IGS_COORD) ?
				     TRUE :
				     FALSE));
	gtk_signal_connect (GTK_OBJECT (check), "toggled",
			    GTK_SIGNAL_FUNC (option_toggled_callback),
			    (gpointer)COORD_STYLE);
	gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, TRUE, 0);
	gtk_widget_show (check);

	/* these ones survive from testgoboard.c :) */
	check = gtk_check_button_new_with_label ("Show Coordinates");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (check),
				     GTK_GO_BOARD (temp_settings->go_board)->show_coordinates);
	gtk_signal_connect (GTK_OBJECT (check), "toggled",
			    GTK_SIGNAL_FUNC (option_toggled_callback),
			    (gpointer)SHOW_COORDINATES);
	gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, TRUE, 0);
	gtk_widget_show (check);
	
	check = gtk_check_button_new_with_label ("High Contrast");
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (check),
				     GTK_GO_BOARD (temp_settings->go_board)->high_contrast);
	gtk_signal_connect (GTK_OBJECT (check), "toggled",
			    GTK_SIGNAL_FUNC (option_toggled_callback),
			    (gpointer)HIGH_CONTRAST);
	gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, TRUE, 0);
	gtk_widget_show (check);
	gtk_widget_show (vbox);

	/* Other options to implement:
	 * - Background pixmap
	 * - Calibrate shiny light on stones
	 * - Skin? Eww.. ;) But yeah, eventually.
	 */

	return frame;
	
}

/************************/
/* Main Settings Dialog */
/************************/

/* settings_dialog
 * open settings dialog to given page
 * using data from given game struct
 * code copied and pasted from inane, then messed about with
 * what crack was i on when i wrote inane?
 */
void settings_dialog(GtkWidget *go_board, struct game *g, gint page)
{

	GtkWidget *window;
	
	GtkWidget *ok_button;
	GtkWidget *save_button;
	GtkWidget *cancel_button;
	
	GtkWidget *notebook;
	
	GtkWidget *display_page;
	GtkWidget *rule_page;
	GtkWidget *player_page;
	GtkWidget *ai_page;
	
	/* make temporary settings */
	make_temp_settings(go_board, g);
	
	/* create dialog widget */
	window = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window), NAME " Options");
	gtk_window_set_default_size(GTK_WINDOW(window), 300, 300);

	/* callback for destroying widget */
	gtk_signal_connect(GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (destroy), (gpointer)window);
	
	/* add a notebook */
	
	notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
	
	/* add pages */
	display_page = make_display_settings (notebook);
	/* only add these pages if we're not playing */
	if(temp_settings->playflag == 0) {
		rule_page = make_rule_settings (notebook);
		player_page = make_player_settings (notebook);
		ai_page = make_ai_settings(notebook);
	}		

	/* pack it and show it */
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox),
			    notebook, TRUE, TRUE, 5);
	gtk_widget_show_all (notebook);
	
	/* set page */
	/* warning - page not tested for sanity... */
	gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
	
	/* ok button */
	ok_button = gtk_button_new_with_label ("Set");
	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			    GTK_SIGNAL_FUNC (options_ok), (gpointer)window);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area),
			    ok_button, FALSE, TRUE, 5);
	gtk_widget_show (ok_button);
	
	/* save button */
	save_button = gtk_button_new_with_label ("Save");
	gtk_signal_connect (GTK_OBJECT (save_button), "clicked",
			    GTK_SIGNAL_FUNC (options_save), (gpointer)window);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area),
			    save_button, FALSE, TRUE, 5);
	gtk_widget_show (save_button);

	/* cancel button */
	cancel_button = gtk_button_new_with_label ("Cancel");
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (kill_window), (gpointer)window);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area),
			    cancel_button, FALSE, TRUE, 5);
	GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (cancel_button);
	gtk_widget_show (cancel_button);
	
	/* display dialog */
	gtk_widget_show (window);
	
	/* grab focus */
	gtk_grab_add (window);
	
}


