/*
 * sgf.c
 */

#include <stdarg.h> /* va_arg macros */
#include <stdlib.h> /* malloc, free */

#include "sgf.h"

#define DATEBUF 12

/* I'm *sure* I shouldn't have to do this */
/* But if I include <time.h> everything goes bezerk */
extern GTime  time(GTime *t);

/* get_yyyymmdd
 * this should *so* not be in this file
 * But I don't know where to put it yet
 */
gchar *get_yyyymmdd(void)
{

	GDate *date;
	static gchar datestr[DATEBUF];

	date = g_date_new();
	g_date_set_time(date, time(NULL));
	g_date_strftime(datestr, DATEBUF, "%Y-%m-%d", date);
	g_date_free(date);

	return datestr;

}

/* init_sgf
 * create and return a new sgf collection
 * with one tree containing one empty node
 * and a default filename
 */
struct sgfcoll *init_sgf(void)
{

	struct sgfcoll *col;

	col = new_sgfcoll(NULL);

	col->num_trees = 1;
	col->tree = new_sgftree(col->tree);
	col->tree->nodes = new_sgfnode(col->tree->nodes, 0);

	col->filename = g_strconcat(g_get_current_dir(), "/goben", get_yyyymmdd(), ".sgf", NULL);

	return col;

}

/* cleanup_sgf
 * frees given col and all associated data
 */
void cleanup_sgf(struct sgfcoll *col)
{

	if(col)
		free_sgfcoll(col);

}



/* set_sgf_header
 * writes following game data to
 * correct tree in game->col
 * FIXME - relatively easy until we support multiple trees, although avoiding
 *         duplicates is hard until we can guarantee that all setup/root/info
 *         stuff goes in the first node
 *         then harder - how can we ensure header goes in right place
 */
gint set_sgf_header(struct game *g)
{

	gint err = 0;

	/* Some basic defaults */
	set_sgf_prop(g->col->tree, SP_FF, 4);
	set_sgf_prop(g->col->tree, SP_GM, 1);
	set_sgf_prop(g->col->tree, SP_AP, NAME ":" VERSION );
	set_sgf_prop(g->col->tree, SP_KM, g->komi);
	set_sgf_prop(g->col->tree, SP_SZ, g->boardsize);
	set_sgf_prop(g->col->tree, SP_PB, g->pname[BLACK]->str);
	set_sgf_prop(g->col->tree, SP_PW, g->pname[WHITE]->str);
	set_sgf_prop(g->col->tree, SP_RU, "Japanese");
	set_sgf_prop(g->col->tree, SP_GC, g->info->str);
	set_sgf_prop(g->col->tree, SP_GN, g_basename(g->col->filename));
	set_sgf_prop(g->col->tree, SP_DT, get_yyyymmdd());

	if(g->handicap > 1) {
		set_sgf_prop(g->col->tree, SP_HA, g->handicap);
		/* FIXME!!! - Add set_sgf_prop(SP_AB) here */
	}

	return err;

}

/* set_sgf_prop
 * adds a property to a node on the given sgftree
 * root or gameinfo properties go on the first node of a tree
 * other properties on the last
 * requires suitable node to have been created
 */
gint set_sgf_prop(struct sgftree *tree, gint prop, ...)
{

	va_list ap;
	gint err = 0;
	gint valtype;
	gint proptype;

	gint x, y;
	gchar *text;

	struct sgfnode *node;
	struct sgfprop *p;
	struct sgfvalue *val;

	node = tree->nodes;

	/* Non root, non gameinfo properties ff to last node */
	proptype = get_proptype(prop);

	/* FIXME - this doesn't let us *add* variations */
	if((proptype == SGFMOVE) ||
	   (proptype == SGFNONE) ||
	   (proptype == SGFSETUP)) {
		while(node && (node->children != NULL)) {
			node = node->children;
		}
	}

	/* Demand that node already exist and be created elsewhere */
	if(!node) {
		g_print("set_sgf_prop: My tree has no node. It smells terrible\n");
		return -1;
	}

	/* Get the properties list for this node */
	p = node->props;

	/* Make sure we have a value to write to */
	if(!p) {
		/* No list? Make one */
		node->props = new_sgfprop(node->props, prop);
		p = node->props;
		/* FIXME - why are we lying to our own function? */
		p->value = new_sgfvalue(p->value, SGFNONE);
		val = p->value;
	} else {
		/* Have list? Search for this prop */
		val = get_sgfvalue(p, prop);
		if(!val) {
			/* no such property there, add a new one */
			node->props = new_sgfprop(node->props, prop);
			p = node->props;
			while(p->next != NULL) {
				p = p->next;
			}
			/* FIXME - why are we lying to our own function? */
			p->value = new_sgfvalue(p->value, SGFNONE);
			val = p->value;
		}
	}

	valtype = get_valtype(prop);
	val->valtype = valtype;

//	g_print("set_sgfprop: prop %d, valtype %d\n", prop, valtype);

        /* Get whatever value was passed in */
	va_start(ap, prop);

	/* Set it */
	switch(valtype) {
	case SGFNONE:
		val->value.none = 0;
		break;
	case SGFNUMBER:
		val->value.number = va_arg(ap, gint);
		break;
	case SGFDOUBLE:
		val->value.sgf_double = va_arg(ap, gint);
		break;
	case SGFCOLOUR:
		val->value.colour = va_arg(ap, gint);
		break;
	case SGFTEXT:
		text = va_arg(ap, gchar *);
		val->value.text = g_strdup(text);
//		g_print("set_sgfprop: added %s\n", val->value.text);
		break;
	case SGFSIMPLETEXT:
		text = va_arg(ap, gchar *);
		val->value.simpletext = g_strdup(text);
//		g_print("set_sgfprop: added %s\n", val->value.simpletext);
		break;
	case SGFREAL:
		val->value.real = va_arg(ap, gdouble);
		break;
	case SGFPOINT:
		x = va_arg(ap, gint);
		y = va_arg(ap, gint);
//		g_print("set_sgfprop - got %d, %d\n", x, y);
		val->value.point = new_sgfpoint(x, y);
		break;
	default:
		g_print("set_sgfprop - warning, unknown valtype %d\n", valtype);
		break;
	};

	va_end(ap);

	return err;

}

/* add_sgf_move
 * add a node to tree
 * add given move to that node
 * return new node
 */
struct sgfnode *add_sgf_move(struct sgftree *tree, gint x, gint y, gint colour)
{

	gint err;
	struct sgfnode *node;

	node = tree->nodes;
	while(node && node->children) {
		node = node->children;
	}
	/* FIXME - can't *add* variations like this */
	node = new_sgfnode(node, 0);

	/* This should 'never happen' */
	if(!tree->nodes) {
		tree->nodes = node;
	}

	if(colour == BLACK) {
		err = set_sgf_prop(tree, SP_B, x, y);
	} else {
		err = set_sgf_prop(tree, SP_W, x, y);
	}
	
	if(err)
		g_warning("add_sgf_move: set_sgf_prop failed\n");


	return node;

}

/* save_sgf
 * wrapper round sgftree.c function
 */
gint save_sgf(struct sgfcoll *col)
{
	gint err;
	err = save_sgfcoll(col);
	return err;
}

/* save_sgf_as
 * sets newfilename, then calls save_sgfcoll
 * assumes filename sanity is checked elsewhere
 */
gint save_sgf_as(struct sgfcoll *col, gchar *newfilename)
{
	gint err;
	col->filename = g_strdup(newfilename);
	err = save_sgfcoll(col);
	return err;
}

/* get_sgf_prop_by_node
 * search a node for a given property
 * return data if found, NULL if not
 */
gpointer get_sgf_prop_by_node(struct sgfnode *node, gint prop)
{

	struct sgfprop *p;
	struct sgfvalue *val;

	gpointer data;

	p = node->props;

	while(p) {
		if(p->property == prop) {
			val = get_sgfvalue(p, prop);
			switch(get_valtype(prop)) {
			case SGFNUMBER:
				data = &val->value.number;
				break;
			case SGFREAL:
				data = &val->value.real;
				break;
			case SGFPOINT:
				/* Handle point lists */
				if(get_listtype(prop) == SL_NONE) {
					data = val->value.point;
				} else {
					data = val;
				}
				break;
			case SGFTEXT:
				data = val->value.text;
				break;
			case SGFSIMPLETEXT:
				data = val->value.simpletext;
				break;
			case SGFDOUBLE:
				data = &val->value.sgf_double;
				break;
			case SGFCOLOUR:
				data = &val->value.colour;
				break;
			default:
				data = &val->value.none;
				break;
			}
			return data;
		}
		p = p->next;
	}

	return NULL;
}

/* get_sgf_prop
 * search a tree for a given property
 * return data if found, NULL if not
 */
gpointer get_sgf_prop(struct sgftree *tree, gint prop)
{

	struct sgfnode *node;
	struct sgfprop *p;
	struct sgfvalue *val;

	gpointer data;

	node = tree->nodes;

	while(node) {

		p = node->props;

		while(p) {
			if(p->property == prop) {
				val = get_sgfvalue(p, prop);
				switch(get_valtype(prop)) {
				case SGFNUMBER:
					data = &val->value.number;
					break;
				case SGFREAL:
					data = &val->value.real;
					break;
				case SGFPOINT:
					/* Handle point lists */
					if(get_listtype(prop) == SL_NONE) {
						data = val->value.point;
					} else {
						data = val;
					}
					break;
				case SGFTEXT:
					data = val->value.text;
					break;
				case SGFSIMPLETEXT:
					data = val->value.simpletext;
					break;
				case SGFDOUBLE:
					data = &val->value.sgf_double;
					break;
				case SGFCOLOUR:
					data = &val->value.colour;
					break;
				default:
					data = &val->value.none;
					break;
				}
				return data;
			}
			p = p->next;
		}
		/* FIXME - variations??? */
		node = node->children;
	}	
	
	return NULL;

}

/* sgf_node_props
 * pass a node and a game struct
 * sets values for comment, nodename and node annotation if found
 */
gint sgf_node_props(struct sgfnode *node, struct game *g)
{

	gint *note;
	gdouble *val;
	gchar *name;
	gchar *comment;
	gint err = 0;

	if(!node) {
		g_print("sgf_node_props: got NULL node\n");
		return 1;
	}

	/* Get comment on last node */
	if((comment = get_sgf_prop_by_node(node, SP_C))) {
		g->comment = g_string_assign(g->comment, comment);
	} else {
		g->comment = g_string_assign(g->comment, "");
	}
	/* Get nodename */
	if((name = get_sgf_prop_by_node(node, SP_N))) {
		g->nodename = g_string_assign(g->nodename, name);
	} else {
		g->nodename = g_string_assign(g->nodename, "");
	}
	/* Get value */
	if((val = get_sgf_prop_by_node(node, SP_V))) {
		g->nodeval = *val;
	} else {
		g->nodeval = 0.0;
	}
	/* Get annotations */
	if((note = get_sgf_prop_by_node(node, SP_DM))) {
		g->nodenote = (*note == 2) ? SN_DM2 : SN_DM1;
	} else if ((note = get_sgf_prop_by_node(node, SP_GB))) {
		g->nodenote = (*note == 2) ? SN_GB2 : SN_GB1;
	} else if ((note = get_sgf_prop_by_node(node, SP_GW))) {
		g->nodenote = (*note == 2) ? SN_GW2 : SN_GW1;
	} else if ((note = get_sgf_prop_by_node(node, SP_HO))) {
		g->nodenote = (*note == 2) ? SN_HO2 : SN_HO1;
	} else if ((note = get_sgf_prop_by_node(node, SP_UC))) {
		g->nodenote = (*note == 2) ? SN_UC2 : SN_UC1;
	} else {
		g->nodenote = SN_NONE;
	}

	return err;

}

/* sgf_move_props
 * pass a node and a game struct
 * sets movenote for any BM, DO, IT, TE properties found
 */
gint sgf_move_props(struct sgfnode *node, struct game *g)
{

	gint err = 0;
	gint *note;

	if(!node) {
		g_print("sgf_move_props: got NULL node\n");
		return 1;
	}

	/* Get annotations */
	if((note = get_sgf_prop_by_node(node, SP_BM))) {
		g->movenote = (*note == 2) ? SM_BM2 : SM_BM1;
	} else if ((note = get_sgf_prop_by_node(node, SP_DO))) {
		g->movenote = SM_DO;
	} else if ((note = get_sgf_prop_by_node(node, SP_IT))) {
		g->movenote = SM_IT;
	} else if ((note = get_sgf_prop_by_node(node, SP_TE))) {
		g->movenote = (*note == 2) ? SM_TE2 : SM_TE1;
	} else {
		g->movenote = SM_NONE;
	}

	return err;
}

/* sgf_setup_props
 * pass it a node and a game struct
 * implements any PL, AB, AE or AW properties found in that node
 */
gint sgf_setup_props(struct sgfnode *node, struct game *g)
{

	gint *colour;
	gint err = 0;
	struct sgfvalue *val;

	if((val = get_sgf_prop_by_node(node, SP_AB))) {
		while(val) {
			err = do_set(&g, val->value.point->x, g->boardsize - val->value.point->y - 1, GOBAN_B);
			if(err)
				return err;
			val = val->next;
		}
	}
	if((val = get_sgf_prop_by_node(node, SP_AW))) {
		while(val) {
			err = do_set(&g, val->value.point->x, g->boardsize - val->value.point->y - 1, GOBAN_W);
			if(err)
				return err;
			val = val->next;
		}
	}
	if((val = get_sgf_prop_by_node(node, SP_AE))) {
		while(val) {
			err = do_set(&g, val->value.point->x, g->boardsize - val->value.point->y - 1, GOBAN_EMPTY);
			if(err)
				return err;
			val = val->next;
		}
	}

	if((colour = get_sgf_prop_by_node(node, SP_PL))) {
		if(*colour == WHITE) {
			g->tomove = GOBAN_W;
		} else if(*colour == BLACK) {
			g->tomove = GOBAN_B;
		} else {
			g_warning("sgf_setup_props: PL got weird colour value '%d'\n", *colour);
		}

	}

	return err;

}

/* sgf_info_props
 * scans sgf from beginning of tree for root and gameinfo properties
 * reads GN, PB, PW, RE, KM, SZ, HA
 * sets game struct accordingly
 * FIXME - should also read RU and TM here
 */
gint sgf_info_props(struct game *g)
{

	gint err = 0;
	gchar   *cdata;
	gint    *idata;
	gdouble *ddata;

	/* get header info:
	 * GN, PB, PW, RE
	 * KM SZ HA
	 */

	/* GN or filename */
	if((cdata = (gchar *)get_sgf_prop(g->col->tree, SP_GN))) {
		g_string_assign(g->info, g_strdup(cdata));
	} else {
		g_string_assign(g->info, g_basename(g->col->filename));
	}

	/* Player names */
	if((cdata = (gchar *)get_sgf_prop(g->col->tree, SP_PB))) {
		g_string_assign(g->pname[BLACK], g_strdup(cdata));
	} else {
		g_string_assign(g->pname[BLACK], GODEF_P1NAME);
	}
	if((cdata = (gchar *)get_sgf_prop(g->col->tree, SP_PW))) {
		g_string_assign(g->pname[WHITE], g_strdup(cdata));
	} else {
		g_string_assign(g->pname[WHITE], GODEF_P2NAME);
	}
	/* Result */
	if((cdata = (gchar *)get_sgf_prop(g->col->tree, SP_RE))) {
		g_string_assign(g->score, g_strdup(cdata));
		g->have_result = 1;
	}
	/* Komi */
	if((ddata = (gdouble *)get_sgf_prop(g->col->tree, SP_KM))) {
		g->komi = *ddata;
	}
	/* Boardsize with sanity check */
	if((idata = (gint *)get_sgf_prop(g->col->tree, SP_SZ))) {
//		idata = (gint *)idata;
		if((*idata < BOARD_MIN) || (*idata > BOARD_MAX)) {
			g_warning("sync_to_sgf: out of range boardsize %d given with SZ, ignoring\n", *idata);
			err = 1;
		} else {
			g->boardsize = *idata;
		}
	} else {
		/* Default to 19 if not given */
		g->boardsize = 19;
	}
	/* Fixed handicap */
	if((idata = get_sgf_prop(g->col->tree, SP_HA))) {
		g->handicap = *idata;
	} else {
		/* Assume missing HA to mean no handicap */
		g->handicap = 0;
	}

	return err;

}

/* get_xtra
 * allocate an xtra info struct
 * fill it with the xtra info properties
 * not read by sgf_info_properties
 */
struct sgfxtra *get_xtra(struct sgftree *tree)
{

	gchar *cdata;
	struct sgfxtra *xtra;

	xtra = malloc(sizeof(struct sgfxtra));

	/* Annotator */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_AN))) {
		xtra->annotator = g_string_new(g_strdup(cdata));
	} else {
		xtra->annotator = NULL;
	}
	/* Copyright */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_CP))) {
		xtra->copyright = g_string_new(g_strdup(cdata));
	} else {
		xtra->copyright = NULL;
	}
	/* Source */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_SO))) {
		xtra->source = g_string_new(g_strdup(cdata));
	} else {
		xtra->source = NULL;
	}
	/* User */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_US))) {
		xtra->user = g_string_new(g_strdup(cdata));
	} else {
		xtra->user = NULL;
	}
	/* Gameinfo */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_GC))) {
		xtra->gameinfo = g_string_new(g_strdup(cdata));
	} else {
		xtra->gameinfo = NULL;
	}
	/* Opening */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_ON))) {
		xtra->opening = g_string_new(g_strdup(cdata));
	} else {
		xtra->opening = NULL;
	}

	/* Teams and ranks */
	/* White team */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_WT))) {
		xtra->team[WHITE] = g_string_new(g_strdup(cdata));
	} else {
		xtra->team[WHITE] = NULL;
	}
	/* White rank */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_WR))) {
		xtra->rank[WHITE] = g_string_new(g_strdup(cdata));
	} else {
		xtra->rank[WHITE] = NULL;
	}
	/* Black team */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_BT))) {
		xtra->team[BLACK] = g_string_new(g_strdup(cdata));
	} else {
		xtra->team[BLACK] = NULL;
	}
	/* Black rank */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_BR))) {
		xtra->rank[BLACK] = g_string_new(g_strdup(cdata));
	} else {
		xtra->rank[BLACK] = NULL;
	}

	/* EVent */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_EV))) {
		xtra->event = g_string_new(g_strdup(cdata));
	} else {
		xtra->event = NULL;
	}
	/* Date */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_DT))) {
		xtra->date = g_string_new(g_strdup(cdata));
	} else {
		xtra->date = NULL;
	}
	/* Place */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_PC))) {
		xtra->place = g_string_new(g_strdup(cdata));
	} else {
		xtra->place = NULL;
	}
	/* Round */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_RO))) {
		xtra->round = g_string_new(g_strdup(cdata));
	} else {
		xtra->round = NULL;
	}
	/* Overtime */
	if((cdata = (gchar *)get_sgf_prop(tree, SP_OT))) {
		xtra->overtime = g_string_new(g_strdup(cdata));
	} else {
		xtra->overtime = NULL;
	}


	return xtra;

}

/* free_xtra
 * cleans up xtra info struct
 */
void free_xtra(struct sgfxtra *xtra)
{

	gint i;

	if(xtra->annotator)
		g_string_free(xtra->annotator, TRUE);

	if(xtra->copyright)
		g_string_free(xtra->copyright, TRUE);
	if(xtra->source)
		g_string_free(xtra->source, TRUE);
	if(xtra->user)
		g_string_free(xtra->user, TRUE);
	if(xtra->gameinfo)
		g_string_free(xtra->gameinfo, TRUE);
	if(xtra->opening)
		g_string_free(xtra->opening, TRUE);

	for(i = 0; i < 2; i++) {
		if(xtra->team[i])
			g_string_free(xtra->team[i], TRUE);
		if(xtra->rank[i])
			g_string_free(xtra->rank[i], TRUE);
	}

	if(xtra->event)
		g_string_free(xtra->event, TRUE);
	if(xtra->date)
		g_string_free(xtra->date, TRUE);
	if(xtra->place)
		g_string_free(xtra->place, TRUE);
	if(xtra->round)
		g_string_free(xtra->round, TRUE);
	if(xtra->overtime)
		g_string_free(xtra->overtime, TRUE);

	free(xtra);

}


/* load_sgf
 * attempts to load sgf file
 * returns error if fails
 * else calls sync to sgf, making sure game struct
 * and sgf file are as they should be
 */
gint load_sgf(struct game *g, gchar *filename)
{

	gint err = 0;
	struct sgfcoll *newcol;

	newcol = new_sgfcoll(filename);

	err = load_sgfcoll(newcol);

	if(err) {
		cleanup_sgf(newcol);
		g_print("load_sgf: load_sgfcoll failed\n");
	} else {
		if(g->col)
			cleanup_sgf(g->col);
		g->col = newcol;
		err = sgf_info_props(g);
		if(err)
			g_print("load_sgf: sgf_info_props failed\n");
	}

	return err;

}

/* SGF navigation routines
 */

/* cull_sgf_moves
 * Used when restarting a game from the middle of an sgf file.
 * assumes g->nummoves > 0, 0 < g->move < g->nummoves
 * removes SGF moves from g->move+1 onwards
 */
void cull_sgf_moves(struct game *g)
{

	gint count;
	struct sgfnode *node;

	count = 0;
	node = g->col->tree->nodes;

	/* Count up to last move made */
	while(node && (count < g->move - 1)) {

		if(get_sgf_prop_by_node(node, SP_B)) {
			count++;
		} else if(get_sgf_prop_by_node(node, SP_W)) {
			count++;
		}
		node = node->children;
	}

	/* Sanity check */
	if(!node) {
		g_warning("cull_sgf_moves: g->move out of sync with sgf data\n");
		return;
	}

	/* Free all remaining moves */
	if(node->children) {
		free_sgfnode(node->children);
		node->children = NULL;
	}

}

/* sgf_get_vars
 * get a node and two gint pointers
 * find how many siblings we have and which one we are in them
 * fill in pointers accordingly
 */
void sgf_get_vars(struct sgfnode *node, gint *this, gint *total)
{

	struct sgfnode *tmp;

	if(!*this) {
		g_warning("sgf_get_vars: *this not set, fudging\n");
		*this = 1;
	}

	*total = 1;
	
	tmp = node;
	while(tmp && tmp->nextsib) {
		(*total)++;
		tmp = tmp->nextsib;
	}
	tmp = node;
	while(tmp && tmp->prevsib) {
		(*total)++;
		(*this)++;
		tmp = tmp->prevsib;
	}

}

/* sgf_movecount
 * returns total number of moves
 * in path containing given node
 */
gint sgf_movecount(struct game *g, struct sgfnode *node)
{

	gint count = 0;
	struct sgfnode *this;

	/* handle trivial case trivially */
	if(!node)
		return 0;

	this = node;

	/* count to end */
	while(node) {
		if(get_sgf_prop_by_node(node, SP_B)) {
			count++;
		} else if(get_sgf_prop_by_node(node, SP_W)) {
			count++;
		}
		node = node->children;
	}

	/* count back to beginning */
	node = this->parent;
	while(node) {
		if(get_sgf_prop_by_node(node, SP_B)) {
			count++;
		} else if(get_sgf_prop_by_node(node, SP_W)) {
			count++;
		}
		node = node->parent;
	}

	return count;

}

/* sgf_ff
 * unconditionally move to last node of given variation
 * resets game struct from sgf data
 * runs through all nodes until there are no more
 * returns last node found
 */
struct sgfnode *sgf_ff(struct game *g, struct sgfnode *node)
{
	gint err = 0;
	gint move;
	struct sgfnode *this = NULL;
	struct sgfpoint *point;

	/* Do nothing if we don't need to */
	if(g->move && (g->num_moves == g->move)) {
		return node;
	}

	if(!node)
		return NULL;

	/* Then run through all the moves, calling do_move */
	while(node) {
		if(sgf_setup_props(node, g)) {
			g_print("sgf_ff: sgf_setup_props failed\n");
		}
		if(g->tomove == GOBAN_B) {
			move = SP_B;
		} else {
			move = SP_W;
		}
		if((point = get_sgf_prop_by_node(node, move))) {
//			move = (move == SP_B) ? SP_W : SP_B;
//			g_print("Trying move %d, %d\n", point->x, point->y);
			err = do_move(&g, point->x, g->boardsize - point->y - 1);
			if (err)
				break; /* break the while loop */
			remove_captured(&g);

		}
		this = node;
		node = node->children;
	}

	if(err)
		g_print("sgf_ff failed - returning %d\n", err);

	/* Get comments, annotations etc on last node */
	err = sgf_node_props(this, g);
	if(err) {
		g_print("sgf_node_props failed\n");
	} 
	err = sgf_move_props(this, g);
	if(err) {
		g_print("sgf_move_props failed\n");
	}

	return this;

}

/* sgf_node_n
 * unconditionally jump to given node of game
 * with correctly setup position
 */
struct sgfnode *sgf_node_n(struct game *g, gint *x, gint *y, struct sgfnode *node)
{
	gint err = 0;
	gint move;
	struct sgfnode *this;
	struct sgfpoint *point = NULL;

	GPtrArray *path;

	/* Build path from beginning to given node */
	path = g_ptr_array_new();
	this = node;
	while(node) {
		g_ptr_array_add(path, node);
		node = node->parent;
	}

	/* reinitialise game struct */
	init_game(&g, 1);

	node = g_ptr_array_index(path, path->len-1);

	while(path->len > 0) {
		if(sgf_setup_props(node, g)) {
			g_print("sgf_move_n: sgf_setup_props failed\n");
		}
		if(g->tomove == GOBAN_B) {
			move = SP_B;
		} else {
			move = SP_W;
		}
		if((point = get_sgf_prop_by_node(node, move))) {
//			move = (move == SP_B) ? SP_W : SP_B;
//			g_print("Trying move %d, %d\n", point->x, point->y);
			err = do_move(&g, point->x, g->boardsize - point->y - 1);
			if (err)
				break; /* break the while loop */
			remove_captured(&g);
		}

		node = g_ptr_array_remove_index(path, path->len-1);
		if(path->len) {
			node = g_ptr_array_index(path, path->len-1);
		}

	}

	/* Get comment, name, annotations on node found, if any */
	err = sgf_node_props(node, g);
	if(err) {
		g_print("sgf_node_props failed\n");
	}
	err = sgf_move_props(node, g);
	if(err) {
		g_print("sgf_move_props failed\n");
	}

	if(point) {
		*x = point->x;
		*y = point->y;
		//		g->move = count;
//		g_print("sgf_move_n: M%d - %d, %d\n", count, *x, *y);

	}

	/* free pointer array */
	g_ptr_array_free(path, FALSE);

	return node;

}

/* sgf_back
 * go back a node
 * return new node
 */
struct sgfnode *sgf_back(struct game *g, gint *x, gint *y, struct sgfnode *node)
{

	gint err;
	*x = -2;
	*y = -2;

	/* ignore null nodes */
	if(!node)
		return NULL;

	if(node->parent) {
		node = sgf_node_n(g, x, y, node->parent);
	} else {
		/* Back to root node */
		init_game(&g, 1);

		/* Get comments and node annotations */
		err = sgf_node_props(node, g);
		if(err) {
			g_print("sgf_node_props failed\n");
		}

	}
		
	/* setup properties may be present */
	if(sgf_setup_props(node, g)) {
		g_print("sgf_rw: sgf_setup_props failed\n");
	}

	return node;

}

/* sgf_nextvar
 * goes to next sibling
 * if NULL, goes to earliest previous sibling
 * if still NULL does nothing
 * takes x, y, values and updates them if move found or not
 */
struct sgfnode *sgf_nextvar(struct game *g, gint *x, gint *y, struct sgfnode *node)
{

	struct sgfnode *tmp;

	*x = -2;
	*y = -2;

	/* should 'never happen' */
	if(!node)
		return NULL;

	if(node->nextsib) {
		node = sgf_node_n(g, x, y, node->nextsib);
		return node;
	}

	tmp = node;
	while(tmp && tmp->prevsib) {
		tmp = tmp->prevsib;
	}

	if(tmp != node) {
		node = sgf_node_n(g, x, y, tmp);
	}

	return node;

}

/* sgf_forward
 * go forward a node
 * return new node
 */
struct sgfnode *sgf_forward(struct game *g, gint *x, gint *y, struct sgfnode *node)
{

	gint err;
	gint move;
	struct sgfpoint *point;

	/* default do-nothing vals for *x and *y */
	*x = -2;
	*y = -2;

	/* ignore null nodes */
	if(!node)
		return NULL;

	if(node->children) {
		/* move */
		node = node->children;
		/* check for setup properties */
		if(sgf_setup_props(node, g)) {
			g_print("sgf_move_n: sgf_setup_props failed\n");
		}
		if(g->tomove == GOBAN_B) {
			move = SP_B;
		} else {
			move = SP_W;
		}
		if((point = get_sgf_prop_by_node(node, move))) {
//			move = (move == SP_B) ? SP_W : SP_B;
//			g_print("Trying move %d, %d\n", point->x, point->y);
			err = do_move(&g, point->x, g->boardsize - point->y - 1);
			if (err) {
				g_print("sgf_forward: do_move didn't like %d,%d\n", point->x, point->y-1);
			}
			remove_captured(&g);

			if(point) {
				*x = point->x;
				*y = point->y;
			}

			/* get move properties */
			err = sgf_move_props(node, g);
			if(err) {
				g_print("sgf_move_props failed\n");
			}

		}

		/* Get comments and node annotations */
		err = sgf_node_props(node, g);
		if(err) {
			g_print("sgf_node_props failed\n");
		}
		
	}

	return node;

}



/* sgf_rw
 * unconditionally move to first node of current tree
 */
struct sgfnode *sgf_rw(struct game *g, struct sgfnode *node)
{

	gint err;
	struct sgfnode *tmp;

	/* don't do anything with NULL nodes */
	if(!node)
		return NULL;

	tmp = node;
	while(tmp && tmp->parent) {
		tmp = tmp->parent;
	}
	node = tmp;

	init_game(&g, 1);

	/* setup properties may be present */
	if(sgf_setup_props(node, g)) {
		g_print("sgf_rw: sgf_setup_props failed\n");
	}

	/* Get comments and node annotations */
	err = sgf_node_props(node, g);
	if(err) {
		g_print("sgf_node_props failed\n");
	}

	/* FIXME - we currently have no way of ensuring that moves in the first
	 * node get split into a subsequent node as per the spec, so currently,
	 * while this code makes sense according to strict FF[4], it will fail until
	 * we implement the other bit if a file does have moves in the first node
	 */

	return node;
}
