/*
 * sgftree.c
 */

/* TODO:
 *
 * This is a naive implementation. To do:
 *
 *  1 - Syntax checking i   - uniqueness of properties in a node
 *  2 - Syntax checking ii  - allowedness of properties in a node
 *  3 - Syntax checking iii - further property specific syntax
 *  4 - Elist unsupported
 *  5 - Composed data unsupported
 *  6 - Composed points unsupported
 *  7 - Composed point:text unsupported
 *  8 - Proper handling of unknown properties unsupported
 */

#include <stdlib.h> /* malloc, free */
#include <stdio.h>  /* fopen, getc */
#include <errno.h>  /* errno */
#include <stdarg.h> /* va_arg macros */

#include "sgftree.h"
#include "game.h"

#define NUMPROPS 69
#define NUMBITS  7

gint propinfo[NUMPROPS][NUMBITS] = {

	/*Move properties */
        /*Val     Length   Ch1    Ch2    Propvalue    Proptype  Takes list    */
	{ SP_B,   1,       'B',   '0',   SGFPOINT,    SGFMOVE,  SL_NONE, },
	{ SP_KO,  2,       'K',   'O',   SGFNONE,     SGFMOVE,  SL_NONE, },
	{ SP_MN,  2,       'M',   'N',   SGFNUMBER,   SGFMOVE,  SL_NONE, },
	{ SP_W,   1,       'W',   '0',   SGFPOINT,    SGFMOVE,  SL_NONE, },
	/* Setup properties */
	{ SP_AB,  2,       'A',   'B',   SGFPOINT,     SGFSETUP, SL_L, },
	{ SP_AE,  2,       'A',   'E',   SGFPOINT,     SGFSETUP, SL_L, },
	{ SP_AW,  2,       'A',   'W',   SGFPOINT,     SGFSETUP, SL_L, },
	{ SP_PL,  2,       'P',   'L',   SGFCOLOUR,    SGFSETUP, SL_NONE, },
	/* Go specific properties */
	{ SP_HA,  2,       'H',   'A',   SGFNUMBER,    SGFINFO,   SL_NONE, },
	{ SP_KM,  2,       'K',   'M',   SGFREAL,      SGFINFO,   SL_NONE, },
	{ SP_TB,  2,       'T',   'B',   SGFPOINT,     SGFNONE,   SL_L, },
	{ SP_TW,  2,       'T',   'W',   SGFPOINT,     SGFNONE,   SL_L, },
	/* Node annotation properties */
        /*Val     Length   Ch1    Ch2    Propvalue      Proptype   Listtype */
	{ SP_C,   1,       'C',   '0',   SGFTEXT,       SGFNOTYPE, SL_NONE,  },
	{ SP_DM,  2,       'D',   'M',   SGFDOUBLE,     SGFNOTYPE, SL_NONE, },
	{ SP_GB,  2,       'G',   'B',   SGFDOUBLE,     SGFNOTYPE, SL_NONE, },
	{ SP_GW,  2,       'G',   'W',   SGFDOUBLE,     SGFNOTYPE, SL_NONE, },
	{ SP_HO,  2,       'H',   'O',   SGFDOUBLE,     SGFNOTYPE, SL_NONE, },
	{ SP_N,   1,       'N',   '0',   SGFSIMPLETEXT, SGFNOTYPE, SL_NONE, },
	{ SP_UC,  2,       'U',   'C',   SGFDOUBLE,     SGFNOTYPE, SL_NONE, },
	{ SP_V,   1,       'V',   '0',   SGFREAL,       SGFNOTYPE, SL_NONE, },
	/* Move annotation properties */
	{ SP_BM,  2,       'B',   'M',   SGFDOUBLE,     SGFMOVE, SL_NONE, },
	{ SP_DO,  2,       'D',   'O',   SGFNONE,       SGFMOVE, SL_NONE, },
	{ SP_IT,  2,       'I',   'T',   SGFNONE,       SGFMOVE, SL_NONE, },
	{ SP_TE,  2,       'T',   'E',   SGFDOUBLE,     SGFMOVE, SL_NONE, },
	/* Markup properties */
        /*Val     Length   Ch1    Ch2    Propvalue      Proptype   Listtype */
	{ SP_AR,  2,       'A',   'R',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_CR,  2,       'C',   'R',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_DD,  2,       'D',   'D',   SGFPOINT,      SGFNONE,   SL_EL },
	{ SP_LB,  2,       'L',   'B',   SGFPOINT,      SGFNONE,   SL_PT },
	{ SP_LN,  2,       'L',   'N',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_MA,  2,       'M',   'A',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_SL,  2,       'S',   'L',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_SQ,  2,       'S',   'Q',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_TR,  2,       'T',   'R',   SGFPOINT,      SGFNONE,   SL_L },
	/* Root properties */
	{ SP_AP,  2,       'A',   'P',   SGFTEXT,       SGFROOT, SL_NONE, },
	{ SP_CA,  2,       'C',   'A',   SGFSIMPLETEXT, SGFROOT, SL_NONE, },
	{ SP_FF,  2,       'F',   'F',   SGFNUMBER,     SGFROOT, SL_NONE, },
	{ SP_GM,  2,       'G',   'M',   SGFNUMBER,     SGFROOT, SL_NONE, },
	{ SP_ST,  2,       'S',   'T',   SGFNUMBER,     SGFROOT, SL_NONE, },
	{ SP_SZ,  2,       'S',   'Z',   SGFNUMBER,     SGFROOT, SL_NONE, },
	/* Game info properties */
	{ SP_AN,  2,       'A',   'N',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_BR,  2,       'B',   'R',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_BT,  2,       'B',   'T',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_CP,  2,       'C',   'P',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_DT,  2,       'D',   'T',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_EV,  2,       'E',   'V',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_GN,  2,       'G',   'N',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_GC,  2,       'G',   'C',   SGFTEXT,       SGFINFO, SL_NONE, },	
	{ SP_ON,  2,       'O',   'N',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_OT,  2,       'O',   'T',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_PB,  2,       'P',   'B',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_PC,  2,       'P',   'C',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_PW,  2,       'P',   'W',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_RE,  2,       'R',   'E',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_RO,  2,       'R',   'O',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_RU,  2,       'R',   'U',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_SO,  2,       'S',   'O',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_TM,  2,       'T',   'M',   SGFREAL,       SGFINFO, SL_NONE, },
	{ SP_US,  2,       'U',   'S',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_WR,  2,       'W',   'R',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	{ SP_WT,  2,       'W',   'T',   SGFSIMPLETEXT, SGFINFO, SL_NONE, },
	/* Timing properties */
	{ SP_BL,  2,       'B',   'L',   SGFREAL,     SGFMOVE, SL_NONE, },
	{ SP_OB,  2,       'O',   'B',   SGFNUMBER,   SGFMOVE, SL_NONE, },
	{ SP_OW,  2,       'O',   'W',   SGFNUMBER,   SGFMOVE, SL_NONE, },
	{ SP_WL,  2,       'W',   'L',   SGFREAL,     SGFMOVE, SL_NONE, },
	/* Misc properties */
	{ SP_FG,  2,       'F',   'G',   SGFSIMPLETEXT, SGFNONE, SL_NONE, },
	{ SP_PM,  2,       'P',   'M',   SGFNUMBER,     SGFNONE, SL_NONE, },
	{ SP_VW,  2,       'V',   'W',   SGFPOINT,      SGFNONE, SL_EL, },
	/* Pre FF[4] properties */
	{ SP_M,  1,       'M',   '0',   SGFPOINT,      SGFNONE,   SL_L },
	{ SP_L,  1,       'L',   '0',   SGFPOINT,      SGFNONE,   SL_L },

};

/*****************
 * SGF UTILITIES *
 *****************/

/* get_sgfvalue
 * give it a list of properties and a property type
 * returns sgfvalue struct for that property type if it exists in list
 * returns NULL if it doesn't exist in list
 */
struct sgfvalue *get_sgfvalue(struct sgfprop *p, gint prop)
{

	struct sgfvalue *val;
	struct sgfprop *ptmp;

	val = NULL;
	ptmp = p;

	while(ptmp) {

		if(ptmp->property == prop) {
			val = ptmp->value;
//			g_print("get_sgfvalue: found property %d\n", prop);
			break;
		}
		ptmp = ptmp->next;
	};
  
	return val;

}

/* get_proptype
 * pass it an int indicating property
 * get int indicating proptype back, or -1 if passed in insane value
 */
gint get_proptype(gint prop)
{

	gint ret = -1;

	if((prop > -1) && (prop < NUMPROPS)) {
		ret = propinfo[prop][5];
	} else {
		g_warning("get_proptype: got odd value %d\n", prop);
	}

	return ret;

}
/* get_valtype
 * pass it an int indicating property
 * get int indicating valtype back, or -1 if passed in insane value
 */
gint get_valtype(gint prop)
{

	gint ret = -1;

	if((prop > -1) && (prop < NUMPROPS)) {
		ret = propinfo[prop][4];
	} else {
		g_warning("get_valtype: got odd value %d\n", prop);
	}

	return ret;

}

/* get_valtype
 * pass it an int indicating property
 * get int indicating listtype back, or -1 if passed in insane value
 */
gint get_listtype(gint prop)
{
	gint ret = -1;

	if((prop > -1) && (prop < NUMPROPS)) {
		ret = propinfo[prop][6];
	} else {
		g_warning("get_listtype: got odd value %d\n", prop);
	}

	return ret;
}

/* get_property
 * pass it a two character array, a number of characters
 * checks against known properties
 * returns property index if known
 * else return -1
 */
gint get_property(gchar *pi, gint length)
{

	gint i;
	
	for(i = 0; i < NUMPROPS; i++) {
		if (length == 1) {
			if((propinfo[i][1] == 1) && (propinfo[i][2] == pi[0])) {
				return i;
			}
		} else {
			if((propinfo[i][1] == 2) &&
			   (propinfo[i][2] == pi[0]) &&
			   (propinfo[i][3] == pi[1])) {
				return i;
			}
		}
	}

	return -1;
}

/**************
 * SGF PARSER *
 **************/

#define READ_MAX 65536

/* read_value
 * gets buffer and open file stream just after [ found
 * reads into buffer until unescaped ] or end of file found
 * or READ_MAX read (no single SGF value should be more than
 * 64K long; I know the spec explicitly calls for unlimited
 * length strings, but I am also concerned about the case where
 * someone does put the entire works of Shakespeare in a C[] value,
 * to see if the program will crash or not. This way, we won't.
 * If it ever gets picked up on as a problem, the value can always
 * be a) raised to something more reasonable (if there were, say
 * a collection of worth-reading SGF files with huge 64K+ comments)
 * and/or b) be made a user-settable value. My bet is that no-one
 * will ever even notice.
 */
gint read_value(GString *buf, FILE *f)
{

	gint eof = 0;
	gint err = 0;
	gint c;
	gint count = 0;
	gint d;
	gint escape = 0;
	
	while(1) {
		c = getc(f);
		count++;

		if(escape)
			escape--;

		switch(c) {
		case EOF:
			eof = 1;
			err = 4;
			g_print("read_value: unexpected end of file\n");
			break;

		case '\f':
			/* Handle DOS newlines */
			d = getc(f);
			if(d != '\r')
				ungetc(d, f);
		case '\r':
			d = getc(f);
			if(d != '\n')
				ungetc(d, f);
		case '\n':
			d = getc(f);
			if(d != '\r')
				ungetc(d, f);
			/* remove escaped newlines of any flavour */
			if(escape)
				break;
			/* convert remaining newline to local brand and keep */
			buf = g_string_append_c(buf, '\n');
			break;
		case '\v':
		case '\t':
			/* Convert tabs to spaces */
			buf = g_string_append_c(buf, ' ');
			break;
		case ']':
			if(escape == 0) {
				ungetc(c, f);
				eof = 1;
				break;
			}
			/* Otherwise fall through */
		case '\\':
			if(escape == 0) {
				/* escape one char */
				escape = 2;
				break;
			}
			/* Fall through again */

		default:
			buf = g_string_append_c(buf, c);
			break;
		};
		if(eof)
			break;
		if(count == READ_MAX) {
			g_warning("read_value: read 64K in single value, stopping here.");
			break;
		}
	}

	return err;

}

/* add_value
 * take a property, an index into propinfo array and a character string
 * try and parse string appropriately for property and add value in
 * return error if can't do it
 * 6 - bad colour value
 * 7 - bad double value
 */
gint add_value(struct sgfprop *p, gint property, GString *buf)
{

	gint x, y;
	gdouble real;
	gint err = 0;
//	gint valtype = propinfo[property][4];
	gint valtype = get_valtype(property);
	gint colour;

	gchar **bits;
	gchar *tmp;

//	g_print("add_value - proptype %d\n", propinfo[property][4]);

	switch(valtype) {
		
	case SGFNUMBER:
		/* FIXME - add sanity check */
		p->value = new_sgfvalue(p->value, valtype, atoi(buf->str));
		break;
	case SGFREAL:
		/* FIXME - add sanity check */
		real = g_strtod(buf->str, NULL);
//		g_print("add_value: got real: %f\n", real);
		p->value = new_sgfvalue(p->value, valtype, real);
		break;
	case SGFDOUBLE:
		if(buf->len > 0) {
			if((buf->str[0] == '1') || (buf->str[0] == '2')) {
				p->value = new_sgfvalue(p->value, valtype, buf->str[0] - '0');
				break;
			}
		}
		g_print("add_value: 7 bad double value\n");
		err = 7;
		break;
	case SGFCOLOUR:
		if(buf->len > 0) {
			if((buf->str[0] == 'B') || (buf->str[0] == 'W')) {
				/* gint colour used to avoid sending gchar through va_args,
				 * since it otherwise promotes it to gint and some systems
				 * don't like this.
				 */
				colour = buf->str[0] == 'B' ? BLACK : WHITE;
				p->value = new_sgfvalue(p->value, valtype, colour);
				break;
			} else if ((buf->str[0] == '1') || (buf->str[0] == '2')) {
				/* Handle old FF colour way */
				colour = buf->str[0] == '1' ? BLACK : WHITE;
				p->value = new_sgfvalue(p->value, valtype, colour);
				break;
			}
		}
		g_print("add_value: 6 bad colour value - '%s'\n", buf->str);
		err = 6;
		break;

	case SGFTEXT:
		/* newline handling is already done by read_value */
		p->value = new_sgfvalue(p->value, valtype, buf->str);
		break;
	case SGFSIMPLETEXT:
		/* Most newline handling done by read_value
		 * Here we just strip remaining \n
		 */
		bits = g_strsplit(buf->str, "\n", 0);
		tmp = g_strjoinv(NULL, bits);
		buf = g_string_assign(buf, tmp);
		p->value = new_sgfvalue(p->value, valtype, buf->str);
		g_strfreev(bits);
		g_free(tmp);
		break;
	case SGFPOINT:
		/* FIXME - should handle compressed lists properly here */
		if((buf->len > 1) && (g_strncasecmp(buf->str, "tt", 2) != 0)) {
			x = buf->str[0] - 'a';
			y = buf->str[1] - 'a';
		} else {
			x = -1;
			y = -1;
		}
//		g_print("add value - %d, %d\n", x, y);
		p->value = new_sgfvalue(p->value, valtype, x, y);
		break;
	case SGFNONE:
		p->value = new_sgfvalue(p->value, valtype);
		break;
	default:
		g_print("add_value: unknown value type %d\n", valtype);
		err = 1;
		break;

	}


	return err;
	
}

/* parse_node
 * gets sgftree struct and open file stream just after ; found
 * reads up to next node and adds it to tree
 * 0 if ok, errors:
 * 4 - premature eof - we shouldn't get eof in here
 * 5 - unexpected list - more than one propvalue when we weren't expecting it
 */
gint parse_node(struct sgfnode *n, FILE *f)
{

	gint eof = 0;
	gint err = 0;
	gint c;

	GString *buf;
	gchar propident[2];
	gint ipi = 0;
	gint property = -1;
	gint have_prop = 0;

	struct sgfprop *p = n->props;
	while(p && (p->next != NULL)) {
		p = p->next;
	}

	buf = g_string_new(NULL);

	while(1) {
		c = getc(f);
		/* ignore lower case characters */
		if((c >= 'a') && (c <= 'z')) {
			continue;
		}
		/* Grab first two upper case characters we see */
		/* We (arguably) oughtn't allow whitespace between them, but we do */
		if((ipi < 2) && (c >= 'A') && (c <= 'Z')) {
			propident[ipi] = (gchar)c;
			ipi++;
			property = get_property(propident, ipi);
			if(property > -1) {
				have_prop = 1;
				if(propinfo[property][1] == 2) { 
//					g_print("parse_node: %c%c\n",
//						propinfo[property][2], propinfo[property][3]);
				} else {
//					g_print("parse_node: %c\n",
//						propinfo[property][2]);
				}
			}
			continue;
		}
		switch(c) {
		case EOF:
			eof = 1;
			err = 4;
			g_print("parse_node: 4 - premature end of file\n");
			break;
		case ';':
		case ')':
		case '(':
			/* next node, new variation, or end of this variation or file */
			ungetc(c, f);
			eof = 1;
			break;
		case '[':
			/* if have prop set, create new property node, unset have_prop */
			if(have_prop && (property > -1)) {
				/* FIXME - we should not blithely add properties to nodes without
				 * checking whether a) whether we already have such a property, and b)
				 * whether such a property is legal at this point
				 */
				n->props = new_sgfprop(n->props, property);
				have_prop = 0;
				if(p != NULL) {
					while(p->next != NULL) {
						p = p->next;
					}
				} else {
					p = n->props;
				}
			}

			/* found next value, read it in */
			err = read_value(buf, f);
			if(err)
				eof = 1;
			break;
		case ']':
			/* found end of value, time to add property to node */
			/* sanity check - if n->props non null, check we're expecting a list */
			if((property > -1) && (p->value != NULL) && (propinfo[property][6] == SL_NONE)) {
				g_print("parse_node: parse error 5 - wasn't expecting list here\n");
				err = 5;
			} else {
				/* FIXME - there are many further ways in which things could be illegal
				 * should add here a call to a check_syntax function, not just blindly add value
				 */
				if(property > -1) {
					err = add_value(p, property, buf);
				} else {
					g_print("parse_node: unknown property %c%c\n", propident[0], propident[1]);
				}
				/* cycle buffer */
				g_string_free(buf, TRUE);
				buf = g_string_new(NULL);

			}

			ipi = 0;

			if(err)
				eof = 1;

			break;
		case ' ':
		case '\n':
		case '\t':
		case '\f':
		case '\r':
		case '\v':
			/* ignore whitespace between values, properties, nodes, sequences and game trees */
			break;
		default:
			g_print("parse_node: 2 - didn't expect this - '%c'\n", c);
			eof = 1;
			err = 2;
			break;
		};

		if(eof)
			break;

	}

	/* clean up the buffer */
	g_string_free(buf, TRUE);

	return err;
	
}

/* parse_sgf
 * give it sgf_collection struct and open file stream
 * does its best to read the file and parse it as sgf
 * returns 0 on success, positive int on errors:
 * 2 - unexpected character found
 * 3 - ; found outside game tree
 * 8 - nesting error - too many )'s
 */
gint parse_sgf(struct sgfcoll *col, FILE *f)
{

	gint err = 0;
	gint c;
	gint eof = 0;
	gint num_trees = 0;
	gint in_tree = 0;
	gint vdepth = 0;

	struct sgftree *tree = NULL;
	struct sgfnode *lastnode = NULL;

	while(1) {
		c = getc(f);

		switch(c) {
		case EOF:
			eof = 1;
			break;
		case '(':
			if(in_tree) {
				/* a new variation */
				vdepth++;

			} else {
				/* a new tree */

				num_trees++;
				col->tree = new_sgftree(col->tree);
				tree = col->tree;
				while(tree && (tree->next != NULL)) {
					tree = tree->next;
				}
				in_tree++;
				tree->nodes = NULL;

			}
			break;

		case ')':
			if(in_tree) {
				/* end of variation or end of tree? */
				vdepth--;
				if(vdepth < 0) {
					/* that was end of tree, not end of var */
					vdepth = 0;
					in_tree--;
					lastnode = NULL; /* debugging the lack of this line was no fun */
				} else {
					/* end of variation, step back up a level */
					while(lastnode->parent && (lastnode->depth != vdepth)) {
						lastnode = lastnode->parent;
					}
				} 

			} else {
				/* nesting error */
				g_print("parse_sgf: found ) while not in tree\n");
				eof = 1;
				err = 8;
			}
			break;
		case ' ':
		case '\n':
		case '\t':
		case '\f':
		case '\r':
		case '\v':
			/* ignore whitespace between values, properties, nodes, sequences and game trees */
			break;
		case ';':
			if(tree && in_tree) {
				/* we found a node */
				lastnode = new_sgfnode(lastnode, vdepth);
				/* If this is the first node in the tree, add it */
				if(!tree->nodes) {
					tree->nodes = lastnode;
				}

				//g_print("parse_sgf: adding data to node %d\n", lastnode->id);

				err = parse_node(lastnode, f);
					if(err) {
					eof = 1;
				}
			} else {
				/* parse error */
				g_print("parse_sgf: parse error 3 - ; found outside game tree\n");
				eof = 1;
				err = 2;
			}
			break;
		default:
			if(in_tree != 1)
				break;
			g_print("parse_sgf: parse error 2 - unexpected character in file\n");
			eof = 1;
			err = 1;
			break;

		};

		if(eof)
			break;

	}

	return err;

}

/************
 * FILE I/O *
 ************/

/* load_sgfcoll
 * open a filehandle or return 1 for file error
 * call parse_sgf
 * return 0 for ok, or the parse error from parse_sgf
 */
gint load_sgfcoll(struct sgfcoll *col)
{

	gint c, err;
	FILE *f;

	err = 0;
	c = 0;

	f = fopen(col->filename, "r");

	if(!f) {
		perror("load_sgfcoll: fopen");
		g_print("load_sgfcoll: error 1 - couldn't open file\n");
		return 1;
	}

	err = parse_sgf(col, f);

	if(fclose(f)) {
		perror("load_sgfcoll: fclose");
		if(err == 0)
			err = 1;
	}

	return err;

}

/* save_sgfcoll
 * Clobbers whatever col->filename points at
 * with its best effort at writing a new SGF file
 */
gint save_sgfcoll(struct sgfcoll *col)
{

	gint c;
	gint x, y;
	gint err = 0;
	FILE *f;

	struct sgftree *tree;
	struct sgfnode *node;
	//	struct sgfnode *varnode = NULL;
	struct sgfprop *prop;
	struct sgfvalue *val;

	GPtrArray *vars;

	vars = g_ptr_array_new();
	tree = col->tree;

	f = fopen(col->filename, "w");
	
	if(!f) {
		perror("save_sgfcoll: fopen");
		return 1;
	}

	while(tree != NULL) {
		fprintf(f, "(");
		node = tree->nodes;
		c = 7;
		while(node != NULL) {

			if(node->nextsib || node->prevsib) {
				fprintf(f, "(");
			}

			fprintf(f, ";");
			prop = node->props;
			while(prop != NULL) {
				if(propinfo[prop->property][1] == 1) {
					fprintf(f, "%c", propinfo[prop->property][2]);
				} else {
					fprintf(f, "%c%c", propinfo[prop->property][2], propinfo[prop->property][3]);
				}
				val = prop->value;
				while(val != NULL) {

					fprintf(f, "[");
					switch(propinfo[prop->property][4]) {
					case SGFNUMBER:
						fprintf(f, "%d", val->value.number);
						break;
					case SGFREAL:
						fprintf(f, "%f", val->value.real);
						break;
					case SGFCOLOUR:
						/* Colour will only ever be ASCII 'B' or 'W',
						 * and we need to avoid sending gchar through va_args
						 */
						if(val->value.colour == BLACK) {
							fprintf(f, "B");
						} else if (val->value.colour == WHITE) {
							fprintf(f, "W");
						} else {
							g_warning("save_sgfcoll: bad colour value %d\n",
								  val->value.colour);
						};
						break;
					case SGFDOUBLE:
						fprintf(f, "%d", val->value.sgf_double);
						break;
					case SGFTEXT:
						fprintf(f, "%s", val->value.text);
						break;
					case SGFSIMPLETEXT:
						if(val->value.simpletext)
							fprintf(f, "%s", val->value.simpletext);
						break;
					case SGFPOINT:
						/* -1,-1=pass, print [], so nothing here */
						if((val->value.point->x == -1) &&
						   (val->value.point->y == -1)) {
							break;
						}
						x = val->value.point->x + 'a';
						y = val->value.point->y + 'a';
						fprintf(f, "%c%c", val->value.point->x + 'a', val->value.point->y + 'a');
						break;
					case SGFNONE:
					default:
						break;
					}; /* switch on proptype */
					fprintf(f, "]");
					val = val->next;
				};
				prop = prop->next;
				//fprintf(f, " ");
			}
			if(node->children) {

				//g_print("save_sgfcoll: found children\n");
				node = node->children;

			} else {

				fprintf(f, ")");
				while(node && !node->nextsib) {
					if(node->prevsib) {
						fprintf(f, ")");
					}
					node = node->parent;
				}
				if(node) {
					node = node->nextsib;
				} 

			}
			c++;
			if(c == 8) {
				fprintf(f, "\n");
				c = 0;
			} else {
				//	fprintf(f, " ");
			}
		};
		tree = tree->next;
	};
	fprintf(f, "\n");

	if(fclose(f)) {
		perror("load_sgfcoll: fclose");
		if(err == 0)
			err = 1;
	}


	return err;

}

/****************************** 
 * ALLOCATORS AND DESTRUCTORS *
 ******************************/

struct sgfpoint *new_sgfpoint(gint x, gint y)
{

	struct sgfpoint *p;

	p = malloc(sizeof(struct sgfpoint));

	p->x = x;
	p->y = y;

	return p;

}

void free_sgfpoint(struct sgfpoint *p)
{

	free(p);

}

/* new_sgfvalue
 * takes list which may be NULL, value type to add, and actual data
 * Uses va_list but expects exactly one further argument after valtype
 * unless valtype is point, in which case we want two
 * va_list is used because we don't know if the type is gint, gdouble or gchar
 * allocates new value with appropriate data, adds to end of list
 * returns new list
 */
struct sgfvalue *new_sgfvalue(struct sgfvalue *list, gint valtype, ...)
{
	gint x, y;
	va_list ap;
	gchar *text;

	struct sgfvalue *val;
	struct sgfvalue *tmp;

	val = malloc(sizeof(struct sgfvalue));

	val->valtype = valtype;
	val->next = NULL;
	val->compose = NULL;


	/* Get whatever value was passed in */
	va_start(ap, valtype);

	switch(valtype) {
	case SGFNONE:
		val->value.none = 0;
		break;
	case SGFNUMBER:
		val->value.number = va_arg(ap, gint);
		break;
	case SGFDOUBLE:
		val->value.sgf_double = va_arg(ap, gint);
		break;
	case SGFCOLOUR:
		val->value.colour = va_arg(ap, gint);
		break;
	case SGFTEXT:
		text = va_arg(ap, gchar *);
		val->value.text = g_strdup(text);
//		g_print("new_sgfvalue: added %s\n", val->value.text);
		break;
	case SGFSIMPLETEXT:
		text = va_arg(ap, gchar *);
		val->value.simpletext = g_strdup(text);
//		g_print("new_sgfvalue: added %s\n", val->value.simpletext);
		break;
	case SGFREAL:
		val->value.real = va_arg(ap, gdouble);
		break;
	case SGFPOINT:
		x = va_arg(ap, gint);
		y = va_arg(ap, gint);
//		g_print("new_sgfvalue - got %d, %d\n", x, y);
		val->value.point = new_sgfpoint(x, y);
		break;
	default:
		g_print("new_sgfvalue - warning, unknown valtype %d\n", valtype);
		break;
	};

	va_end(ap);

	/* Add new value struct to list */
	if(list == NULL) {
		val->prev = NULL;
		return val;
	}

	tmp = list;

	while(tmp->next != NULL) {
		tmp = tmp->next;
	}

	tmp->next = val;
	val->prev = tmp;

	return list;
}

/* free_sgfvalue
 * frees given sgfvalue and any other sgfvalue in list
 */
void free_sgfvalue(struct sgfvalue *list)
{

	struct sgfvalue *tmp;

	while(list) {
		tmp = list;
		list = tmp->next;

		if(tmp->compose) {
			free_sgfvalue(tmp->compose);
		}

		switch(tmp->valtype) {
		case SGFTEXT:
			g_free(tmp->value.text);
			break;
		case SGFSIMPLETEXT:
			g_free(tmp->value.simpletext);
			break;
		case SGFPOINT:
			free_sgfpoint(tmp->value.point);
			break;
		default:
			break;
		};

		free(tmp);
	}

}

/* new_sgfprop
 * allocates a new sgfprop
 * adds it on end of given list, which may be NULL
 * returns new list
 */
struct sgfprop *new_sgfprop(struct sgfprop *list, gint property)
{

	struct sgfprop *tmp;
	struct sgfprop *prop;

	prop = malloc(sizeof(struct sgfprop));
	prop->property = property;
	prop->value = NULL;
	prop->next = NULL;

	if(list == NULL) {
		prop->prev = NULL;
		return prop;
	}

	tmp = list;

	while(tmp->next != NULL) {
		tmp = tmp->next;
	}

	tmp->next = prop;
	prop->prev = tmp;

	return list;


}


/* free_sgfprop
 * frees given sgfprop and any other sgfprops in list
 */
void free_sgfprop(struct sgfprop *list)
{

	struct sgfprop *tmp;

	while(list) {
		tmp = list;
		list = tmp->next;
		if(tmp->value) {
			free_sgfvalue(tmp->value);
		}
		free(tmp);
	}

}

/* new_sgfnode
 * allocates a new sgfnode
 * adds it as last new child of given node tree, which may be NULL
 * returns new node created
 */
struct sgfnode *new_sgfnode(struct sgfnode *parent, gint depth)
{

	static gint count = 0;
	struct sgfnode *tmp1;
	struct sgfnode *tmp2;
	struct sgfnode *node;

	node = malloc(sizeof(struct sgfnode));
	node->props = NULL;
	node->sibs = 0;
	node->depth = depth;

	/* This is a new node, so no children, no next sibling */
	node->children = NULL;
	node->nextsib = NULL;

	/* id is mainly for debugging purposes */
	node->id = count;

	/* Parent is what was passed in */
	node->parent = parent;

	/* Set new prevsib, parents children or nextsib */
	if(parent) { 
		tmp1 = parent->children;
		if(!tmp1) {
			parent->children = node;
			node->prevsib = NULL;

		} else {
			while(tmp1) {
				tmp2 = tmp1;
				tmp1 = tmp1->nextsib;
				node->sibs++;
			}
			node->prevsib = tmp2;
			tmp2->nextsib = node;
		}
	} else {
		node->prevsib = NULL;
	}

	count++;

	/* return new node */
	//g_print("new_sgfnode: node %d at depth %d has %d siblings\n", node->id, node->depth, node->sibs);
	return node;

}

/* free_sgfnode
 * frees given sgfnode and any other sgfnodes in nodetree
 * and any properties or variations it may contain
 */
void free_sgfnode(struct sgfnode *nodetree)
{

	struct sgfnode *tmp;

	while(nodetree) {
		if(nodetree->props) {
			free_sgfprop(nodetree->props);
		}
		if(nodetree->nextsib) {
			free_sgfnode(nodetree->nextsib);
		}
		tmp = nodetree;
		nodetree = tmp->children;
		free(tmp);
	}

}

/* new_sgftree
 * allocates a new sgftree
 * adds it on end of given list, which may be NULL
 * returns new list
 */
struct sgftree *new_sgftree(struct sgftree *list)
{
	
	struct sgftree *tmp;
	struct sgftree *tree;

	tree = malloc(sizeof(struct sgftree));
	tree->nodes = NULL;
	tree->next = NULL;

	if(!list) {
		tree->prev = NULL;
		return tree;
	}

	tmp = list;

	while(tmp->next) {
		tmp = tmp->next;
	}

	tmp->next = tree;
	tree->prev = tmp;

	return list;

}

/* free_sgftree
 * frees given sgftree and any other sgftrees in list
 */
void free_sgftree(struct sgftree *list)
{

	struct sgftree *tmp;

	while(list) {
		if(list->nodes) {
			free_sgfnode(list->nodes);
		}
		tmp = list;
		list = tmp->next;
		free(tmp);
	}

}

/* new_sgfcoll
 * sets given filename, can be given NULL
 * sets everything else NULL
 * returns newly allocated sgfcoll pointer
 */
struct sgfcoll *new_sgfcoll(gchar *filename)
{

	struct sgfcoll *c;

	c = malloc(sizeof(struct sgfcoll));
	
	if(filename)
		c->filename = g_strdup(filename);
	else
		c->filename = NULL;

	c->num_trees = 0;
	c->tree = NULL;

	return c;

}

/* free_sgfcoll
 */
void free_sgfcoll(struct sgfcoll *c)
{

	if(c->filename)
		g_free(c->filename);

	if(c->tree) {
		free_sgftree(c->tree);
	}

	free(c);

}

/****************
 * TEST ROUTINE *
 ****************/

/* To compile, use gcc -o sgftest sgftree.c -DSGFTREE `glib-config --cflags` `glib-config --libs` */

#ifdef SGFTREE

#define SGFTEST_OUT "./out.sgf"

/* main function
 * compile standalone to test
 * Usage: sgftest <filename>
 * reads file, writes it out again to SGFTEST_OUT
 */
int main ( int argc, char *argv[])
{

	gint err;
	struct sgfcoll *c;

	if(argc != 2) {
		g_print("Usage: sgftest <filename>\n");
		return 1;
	} 

	c = new_sgfcoll(argv[1]);

	g_print("sgftest: Loading %s\n", argv[1]);
	err = load_sgfcoll(c);

	c->filename = g_strdup(SGFTEST_OUT);
	g_print("sgftest: Saving %s\n", c->filename);
	err = save_sgfcoll(c);

	g_print("sgftest: Freeing data\n");
	free_sgfcoll(c);

	g_print("sgftest: Ok\n");

	return err;
}

#endif
