/*
 * sgftree.h
 */

#ifndef __SGFTREE_H__
#define __SGFTREE_H__

#include <glib.h>

struct sgfpoint {
	gint x;
	gint y;
};

enum sgfvaltypes { SGFNONE, SGFNUMBER, SGFREAL, SGFDOUBLE,
		   SGFCOLOUR, SGFSIMPLETEXT, SGFTEXT, SGFPOINT};

struct sgfvalue {
	gint valtype;
	union {
		gint none;
		gint number;
		gdouble real;
		gint sgf_double;
		gint colour;
		gchar *simpletext;
		gchar *text;
		struct sgfpoint *point;
	} value;
	struct sgfvalue *compose;
	struct sgfvalue *next;
	struct sgfvalue *prev;
};

/* FF[4] has 67 (count 'em) properties.
 * That's a lot of properties.
 * Plus 2 recommended old ones.
 *
 * NB - The order of the enum below and the order of the propinfo array in sgftree.c
 *      must be the same, or it will all go horribly wrong.
 */

enum sgfprops {
	/* Move properties */
	SP_B,	SP_KO,	SP_MN,	SP_W,
	/* Setup properties */
	SP_AB,	SP_AE,	SP_AW,	SP_PL,
	/* Go-specific properties */
	SP_HA, SP_KM, SP_TB, SP_TW,
        /* Node annotations */
	SP_C,	SP_DM,	SP_GB,	SP_GW,	SP_HO,	SP_N,	SP_UC,	SP_V,
	/* Move annotations */
	SP_BM,	SP_DO,	SP_IT,	SP_TE,
	/* Markup properties */
	SP_AR,	SP_CR,	SP_DD,	SP_LB,	SP_LN,	SP_MA,	SP_SL,	SP_SQ,	SP_TR,
	/* Root properties */
	SP_AP,	SP_CA,	SP_FF,	SP_GM,	SP_ST, 	SP_SZ,
	/* Game info properties */
	SP_AN, SP_BR, SP_BT, SP_CP, SP_DT, SP_EV, SP_GN, SP_GC, SP_ON, SP_OT,
	SP_PB, SP_PC, SP_PW, SP_RE, SP_RO, SP_RU, SP_SO, SP_TM, SP_US, SP_WR, SP_WT,
	/* Timing properties */
	SP_BL, SP_OB, SP_OW, SP_WL,
	/* Misc properties */
	SP_FG, SP_PM, SP_VW,
	/* Frequent pre FF[4] properties */
	SP_M, SP_L,
};
	
enum proptypes { SGFMOVE, SGFSETUP, SGFROOT, SGFINFO, SGFNOTYPE };
enum listtypes { SL_NONE, SL_L, SL_EL, SL_PT, };

struct sgfprop {
	gint property;
	struct sgfvalue *value;
	struct sgfprop *next;
	struct sgfprop *prev;
};



struct sgfnode {
	gint sibs;
	gint id;
	gint depth;
	struct sgfprop *props;
	struct sgfnode *children;
	struct sgfnode *parent;
	struct sgfnode *nextsib;
	struct sgfnode *prevsib;
};



struct sgftree {
	struct sgfnode *nodes;
	struct sgftree *next;
	struct sgftree *prev;
};



struct sgfcoll {
	gchar *filename;
	gint  num_trees;
	struct sgftree *tree;
};


/* IO */
gint load_sgfcoll(struct sgfcoll *coll);
gint save_sgfcoll(struct sgfcoll *coll);

/* Utilities */
struct sgfvalue *get_sgfvalue(struct sgfprop *p, gint prop);
gint get_valtype(gint prop);
gint get_proptype(gint prop);
gint get_listtype(gint prop);

/* Initialisers/Destructors */
struct sgfpoint *new_sgfpoint(gint x, gint y);
void free_sgfpoint(struct sgfpoint *p);

struct sgfvalue *new_sgfvalue(struct sgfvalue *list, gint valtype, ...);
void free_sgfvalue(struct sgfvalue *list);

struct sgfprop *new_sgfprop(struct sgfprop *list, gint property);
void free_sgfprop(struct sgfprop *list);

struct sgfnode *new_sgfnode(struct sgfnode *parent, gint depth);
void free_sgfnode(struct sgfnode *list);

struct sgftree *new_sgftree(struct sgftree *list);
void free_sgftree(struct sgftree *list);

struct sgfcoll *new_sgfcoll(gchar *filename);
void free_sgfcoll(struct sgfcoll *coll);

#endif /* __SGFTREE_H__ */
